package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.*;
import android.graphics.Color;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.provider.Settings;
import android.view.*;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

/**
 * Created with IntelliJ IDEA.
 * User: Malin
 * Date: 31.01.13
 */
public class CreateActivity extends RoboActivity {

    @InjectView(R.id.EditText_PuzzleName)
    EditText editPuzzleName;
    @InjectView(R.id.Button_Create_Next)
    Button addContentButton;
    @InjectView(R.id.Button_AddFriend_Puzzle)
    Button addFriendButton;
    @InjectView(R.id.Button_Publish)
    Button publishButton;
    @InjectView(R.id.Spinner_Category)
    Spinner spinner;
    @InjectView(R.id.TextView_Number_Questions)
    TextView numberQuestions;
    @InjectView(R.id.TextView_Collaborators)
    TextView collaborators;
    @InjectView(R.id.TableLayout_Friends_Added)
    TableLayout friendsTable;

    private static final int ADD_FRIEND = 1;
    public static final int PUBLISH_PUZZLE = 2;
    public static final int COLLABORATE_PUZZLE = 3;
    private static final int NEED_INTERNET_DIALOG = 4;
    public static final int GET_PUZZLE_TO_EDIT = 9;

    private SharedPreferences settings;
    private ContentHelper helper;
    private ArrayList<ContentValues> questionSet;
    private boolean isMainAuthor;
    private boolean receiverRegistered = false;
    private boolean newlyCreated = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.create);
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        helper = new ContentHelper(this);
        receiverRegistered = false;

        // Populate Spinner control with categories
        ArrayAdapter<?> adapter = ArrayAdapter.createFromResource(this, R.array.categories, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);

        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            // From Create Button extras must be null or at least not contain key
            final int puzzleId = extras.containsKey(General.ID) ? extras.getInt(General.ID) : 0;
            if (puzzleId != 0) {
                newlyCreated = false;
                ContentValues puzzle = helper.getCreatedPuzzle(puzzleId);
                editPuzzleName.setText(puzzle.getAsString(General.PUZZLE_NAME));
                initSpinner(puzzle);

                SharedPreferences.Editor editor = settings.edit();
                editor.putInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, puzzleId);
                editor.commit();

                isMainAuthor = helper.isMainAuthor(puzzleId);
                ArrayList<String> coAuthors = helper.getCoAuthors(puzzleId);

                if (coAuthors.size() != 0) {
                    collaborators.setVisibility(View.VISIBLE);
                    friendsTable.setVisibility(View.VISIBLE);
                    IntentFilter intentFilter = new IntentFilter();
                    intentFilter.addAction(UpdaterService.GET_PUZZLE_EDIT_DONE);
                    registerReceiver(uploadReceiver, intentFilter);
                    receiverRegistered = true;
                    Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                    intent.putExtra(General.UPDATER_ACTION, GET_PUZZLE_TO_EDIT);
                    intent.putExtra(General.ID, puzzleId);
                } else {
                    questionSet = helper.getQuestionSet(settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
                }
                for (String author : coAuthors)
                    insertPlainRow(friendsTable, author);

                if (coAuthors.size() == 0 && (questionSet == null || questionSet.size() == 0)) {
                    newlyCreated = true;
                }

            } else {
                questionSet = new ArrayList<ContentValues>();
                isMainAuthor = true;
            }
        } else {
            questionSet = new ArrayList<ContentValues>();
            isMainAuthor = true;
        }
        numberQuestions.setText(" " + String.valueOf(questionSet == null ? "0" : questionSet.size()));
        addContentButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (editPuzzleName.getText().length() > 5) {
                    int puzzleId = getInfoAndSave();
                    if (friendsTable.getChildCount() > 0 && isMainAuthor)
                        makePuzzleCollaborating(puzzleId);
                    Intent intent = new Intent(getBaseContext(), QuestionSetActivity.class);
                    if (questionSet == null) {
                        questionSet = new ArrayList<ContentValues>();
                    }
                    intent.putParcelableArrayListExtra(General.NEW_QUESTIONSET, questionSet);
                    intent.putExtra(General.COLLABORATE, friendsTable.getChildCount() > 0 || !isMainAuthor);
                    startActivity(intent);
                    finish();
                } else {
                    Toast.makeText(CreateActivity.this, "Fill in the fields properly", Toast.LENGTH_SHORT).show();
                }
            }
        });
        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        final boolean notConnectedToTheInternet = !(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable() && conMgr.getActiveNetworkInfo().isConnected());

        if (isMainAuthor) {
            if (questionSet != null && questionSet.size() != 0) {
                publishButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        showDialog(PUBLISH_PUZZLE);
                    }
                });
            } else {
                addFriendButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        if (!notConnectedToTheInternet)
                            showDialog(ADD_FRIEND);
                        else
                            Toast.makeText(getBaseContext(), R.string.collaborate_need_internet, Toast.LENGTH_SHORT).show();
                    }
                });
                publishButton.setVisibility(View.INVISIBLE);
            }
        } else {
            addFriendButton.setVisibility(View.INVISIBLE);
            publishButton.setVisibility(View.INVISIBLE);
        }
    }

    private void makePuzzleCollaborating(int puzzleId) {
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UpdaterService.COLLABORATE_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        receiverRegistered = true;
        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
        intent.putExtra(General.NEWLY_CREATED, newlyCreated);
        intent.putExtra(General.UPDATER_ACTION, COLLABORATE_PUZZLE);
        intent.putExtra(General.ID, puzzleId);
        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        // ARE WE CONNECTED TO THE INTERNET?
        if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                && conMgr.getActiveNetworkInfo().isConnected())) {
            showDialog(NEED_INTERNET_DIALOG);
        } else {
            startService(intent);
        }
    }

    private int getInfoAndSave() {
        ContentValues puzzleAttributes = new ContentValues();
        puzzleAttributes.put(General.ID, settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
        puzzleAttributes.put(General.PUZZLE_NAME, editPuzzleName.getText().toString());
        puzzleAttributes.put(General.PUZZLE_CATEGORY, spinner.getSelectedItem().toString());
        puzzleAttributes.put(General.XML_TAG_NUMBER_OF_QUESTIONS, questionSet == null ? 0 : questionSet.size());

        ArrayList<String> authors = new ArrayList<String>();
        authors.add(settings.getString(General.GAME_PREFERENCES_NICKNAME, ""));
        for (int i = 0; i < friendsTable.getChildCount(); i += 2) {
            TableRow child = (TableRow) friendsTable.getChildAt(i);
            TextView view = (TextView) child.findViewById(R.id.TextView_Collaborator);
            String friend = view.getText().toString();
            authors.add(friend);
        }
        int puzzleId = helper.savePuzzleToEdit(puzzleAttributes, authors, null, settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
        SharedPreferences.Editor editor = settings.edit();
        editor.putInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, puzzleId);
        editor.commit();
        return puzzleId;
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // Handle the back button from MainCreate to MainActivity
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (editPuzzleName.getText().length() > 2) {
                int puzzleId = getInfoAndSave();
                if (friendsTable.getChildCount() > 0 && isMainAuthor)
                    makePuzzleCollaborating(puzzleId);
                SharedPreferences.Editor editor = settings.edit();
                 editor.remove(General.GAME_PREFERENCES_CURRENT_PUZZLE);
                editor.commit();
                Toast.makeText(getBaseContext(), R.string.saved, Toast.LENGTH_SHORT).show();
            }
            startActivity(new Intent(getBaseContext(), MainActivity.class));
            finish();
            return true;
        } else {
            return super.onKeyDown(keyCode, event);
        }
    }

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case PUBLISH_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.published, Toast.LENGTH_SHORT).show();
                    break;
                case COLLABORATE_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.collaborated, Toast.LENGTH_SHORT).show();
                    break;
                case QuestionSetActivity.QUESTION_SET_UPDATE:
                    Toast.makeText(getBaseContext(), R.string.questionSet_updated, Toast.LENGTH_SHORT).show();
                    break;
                case GET_PUZZLE_TO_EDIT:
                    questionSet = helper.getQuestionSet(settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
                    numberQuestions.setText(" " + String.valueOf(questionSet.size()));
                    /*
                        Collections.sort(questionSet, new Comparator<ContentValues>() {
                            @Override
                            public int compare(ContentValues contentValues, ContentValues contentValues2) {
                                Integer setId = contentValues.getAsInteger(General.XML_TAG_SET_ID);
                                Integer setId2 = contentValues2.getAsInteger(General.XML_TAG_SET_ID);
                                return setId.compareTo(setId2);
                            }
                        });
                     */
                    Toast.makeText(getBaseContext(), R.string.questionSet_updated, Toast.LENGTH_SHORT).show();
                    break;
            }
        }
    };

    @Override
    protected Dialog onCreateDialog(int id) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        switch (id) {
            case ADD_FRIEND:
                View view = getLayoutInflater().inflate(R.layout.include_friend, (ViewGroup) findViewById(R.id.Include_Friend));
                builder.setView(view);
                builder.setIcon(android.R.drawable.ic_input_add);
                TableLayout layout = (TableLayout) view.findViewById(R.id.TableLayout_Include_Friends);
                fillTableLayout(layout);
                builder.setTitle(R.string.include_friend);
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                    }
                });
                return builder.create();
            case PUBLISH_PUZZLE:
                builder.setMessage(R.string.publish_puzzle_message);
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                    }
                });
                builder.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        IntentFilter intentFilter = new IntentFilter();
                        intentFilter.addAction(UpdaterService.PUBLISH_PUZZLE_DONE);
                        registerReceiver(uploadReceiver, intentFilter);
                        receiverRegistered = true;
                        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                        intent.putExtra(General.UPDATER_ACTION, PUBLISH_PUZZLE);
                        intent.putExtra(General.ID, settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
                        // Never been uploaded or not
                        intent.putExtra(General.LOCAL_PUZZLE, !(friendsTable.getChildCount() > 0));

                        getInfoAndSave();
                        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
                        // ARE WE CONNECTED TO THE INTERNET??
                        if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                                && conMgr.getActiveNetworkInfo().isConnected())) {
                            showDialog(NEED_INTERNET_DIALOG);
                            dialogInterface.dismiss();
                        } else {
                            startService(intent);
                            startActivity(new Intent(getBaseContext(), MainActivity.class));
                        }
                    }
                });
                return builder.create();
            case NEED_INTERNET_DIALOG:
                builder.setMessage(R.string.need_internet);
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                        startActivity(new Intent(getBaseContext(), MainActivity.class));
                    }
                });
                builder.setPositiveButton(R.string.to_settings, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
                    }
                });
                return builder.create();
        }
        return null;
    }

    private void fillTableLayout(TableLayout layout) {
        ArrayList<String> friends = helper.getFriends();
        for (String nickname : friends) {
            insertRow(layout, nickname, true);
        }
    }

    private void insertRow(final TableLayout layout, final String nickname, boolean onclick) {
        final TableRow newRow = new TableRow(this);
        final View v = getLayoutInflater().inflate(R.layout.collaborator_entry, newRow, true);
        newRow.setTag(nickname);
        TextView textView = (TextView) v.findViewById(R.id.TextView_Collaborator);
        ImageButton button = (ImageButton) v.findViewById(R.id.ImageButton_DeleteCollaborator);
        textView.setText(nickname);
        textView.setTag(nickname);
        if (onclick) {
            button.setVisibility(View.INVISIBLE);
            newRow.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    addAuthor(nickname);
                    dismissDialog(ADD_FRIEND);
                    friendsTable.setVisibility(View.VISIBLE);
                    collaborators.setVisibility(View.VISIBLE);
                }
            });
        } else {
            button.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    layout.removeView(newRow);
                }
            });
        }
        layout.addView(newRow);
        View line = new View(this);
        line.setLayoutParams(new TableRow.LayoutParams(TableRow.LayoutParams.FILL_PARENT, 1));
        line.setBackgroundColor(Color.rgb(51, 51, 51));
        layout.addView(line);
    }

    private void insertPlainRow(final TableLayout layout, final String nickname) {
        final TableRow newRow = new TableRow(this);
        final View v = getLayoutInflater().inflate(R.layout.collaborator_entry, newRow, true);
        TextView textView = (TextView) v.findViewById(R.id.TextView_Collaborator);
        textView.setText(nickname);
        layout.addView(newRow);
        View line = new View(this);
        line.setLayoutParams(new TableRow.LayoutParams(TableRow.LayoutParams.FILL_PARENT, 1));
        line.setBackgroundColor(Color.rgb(51, 51, 51));
        layout.addView(line);
    }

    private void addAuthor(String nickname) {
        insertRow(friendsTable, nickname, false);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        if (friendsTable.getChildCount() > 0 || !isMainAuthor) {
            getMenuInflater().inflate(R.menu.create_options, menu);
            menu.findItem(R.id.update_menu_item).setIntent(new Intent(this, CreateActivity.class));
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        super.onOptionsItemSelected(item);
        Intent intent = item.getIntent();
        if (item.getItemId() == R.id.update_menu_item) {
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.QUESTION_SET_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            receiverRegistered = true;
            intent.putExtra(General.UPDATER_ACTION, QuestionSetActivity.QUESTION_SET_UPDATE);
            intent.putExtra(General.ID, settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
            ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
            // ARE WE CONNECTED TO THE INTERNET??
            if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                    && conMgr.getActiveNetworkInfo().isConnected())) {
                showDialog(NEED_INTERNET_DIALOG);
            } else {
                startService(intent);
            }
        }
        return true;
    }

    private void initSpinner(ContentValues puzzle) {
        String inputMethod = puzzle.getAsString(General.PUZZLE_CATEGORY);
        if (inputMethod.equals(getResources().getString(R.string.math))) {
            spinner.setSelection(0);
        } else if (inputMethod.equals(getResources().getString(R.string.physics))) {
            spinner.setSelection(1);
        } else if (inputMethod.equals(getResources().getString(R.string.chemistry))) {
            spinner.setSelection(2);
        } else if (inputMethod.equals(getResources().getString(R.string.engineering))) {
            spinner.setSelection(3);
        } else if (inputMethod.equals(getResources().getString(R.string.computer))) {
            spinner.setSelection(4);
        } else if (inputMethod.equals(getResources().getString(R.string.nature))) {
            spinner.setSelection(5);
        } else if (inputMethod.equals(getResources().getString(R.string.psychology))) {
            spinner.setSelection(6);
        } else if (inputMethod.equals(getResources().getString(R.string.english))) {
            spinner.setSelection(7);
        } else if (inputMethod.equals(getResources().getString(R.string.expressions))) {
            spinner.setSelection(8);
        } else if (inputMethod.equals(getResources().getString(R.string.other))) {
            spinner.setSelection(9);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

}
