package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.*;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class FriendsActivity extends RoboActivity {
    @InjectView(R.id.TableLayout_Friends)
    TableLayout friendsLayout;
    @InjectView(R.id.Button_addFriend)
    Button addFriendButton;

    private static final int ADD_FRIEND_DIALOG = 1;
    public static final int FRIEND_ADDED = 13;
    public static final int GET_FRIENDS = 17;
    private ContentHelper helper;
    private boolean receiverRegistered = false;

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            ArrayList<String> friends = helper.getFriends();
            if (action == FRIEND_ADDED) {
                String response = intent.getStringExtra(General.RESPONSE);
                if(response == null){
                    response = getResources().getString(R.string.friend_added);
                }
                Toast.makeText(getBaseContext(), response, Toast.LENGTH_SHORT).show();
                processFriends(friends);
            } else if(action == GET_FRIENDS){
                processFriends(friends);
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.friends);
        helper = new ContentHelper(this);

        addFriendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                showDialog(ADD_FRIEND_DIALOG, null);
            }
        });

        ArrayList<String> friends = helper.getFriends();
        if(friends.size() == 0){
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.GET_FRIENDS_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            receiverRegistered = true;
            Intent intent = new Intent(getBaseContext(), UpdaterService.class);
            intent.putExtra(General.UPDATER_ACTION, GET_FRIENDS);
            startService(intent);
        }
        processFriends(friends);
    }

    private void initializeHeaderRow(TableLayout scoreTable) {
        // Create the Table header row
        TableRow headerRow = new TableRow(this);
        int textColor = getResources().getColor(R.color.app_text_color);
        float textSize = getResources().getDimension(R.dimen.thirteendp);
        addTextToRowWithValues(headerRow, getResources().getString(R.string.nickname), textColor, textSize);
        scoreTable.addView(headerRow);
    }

    private void processFriends(ArrayList<String> nicknames) {
        friendsLayout.removeAllViews();
        initializeHeaderRow(friendsLayout);
        for (String nickname : nicknames) {
            insertFriendRow(nickname);
        }
        if (nicknames.size() == 0)
            insertFriendRow(getResources().getString(R.string.no_friends));
    }

    private void insertFriendRow(String nickname) {
        View line = new View(this);
        line.setLayoutParams(new TableRow.LayoutParams(TableRow.LayoutParams.FILL_PARENT, 1));
        line.setBackgroundColor(Color.rgb(51, 51, 51));
        friendsLayout.addView(line);

        final TableRow newRow = new TableRow(FriendsActivity.this);
        int textColor = getResources().getColor(R.color.app_text_color);
        float textSize = getResources().getDimension(R.dimen.tendp);
        addTextToRowWithValues(newRow, nickname, textColor, textSize);
        friendsLayout.addView(newRow);
    }

    private void addTextToRowWithValues(final TableRow tableRow, String text, int textColor, float textSize) {
        TextView textView = new TextView(getBaseContext());
        textView.setTextSize(textSize);
        textView.setTextColor(textColor);
        textView.setText(text);
        textView.setPadding(5, 10, 5, 10);
        tableRow.addView(textView);
    }

    @Override
    protected Dialog onCreateDialog(int id, Bundle bundle) {
        switch (id) {
            case ADD_FRIEND_DIALOG:
                View layout = getLayoutInflater().inflate(R.layout.friend_entry, (ViewGroup) findViewById(R.id.Friend_Dialog));
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setView(layout);
                builder.setTitle(R.string.add_friend);
                builder.setIcon(android.R.drawable.ic_dialog_info);
                final TextView field = (TextView) layout.findViewById(R.id.EditText_Friend_Field);
                builder.setPositiveButton(R.string.save, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        IntentFilter intentFilter = new IntentFilter();
                        intentFilter.addAction(UpdaterService.FRIEND_ADDED_DONE);
                        registerReceiver(uploadReceiver, intentFilter);
                        receiverRegistered = true;

                        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                        intent.putExtra(General.XML_TAG_NICKNAME, field.getText().toString());
                        intent.putExtra(General.UPDATER_ACTION, FRIEND_ADDED);
                        startService(intent);
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                    }
                });
                return builder.create();
        }
        return null;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }
}
