package com.master.playit;

import android.content.*;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.view.View;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class MainPlayActivity extends RoboActivity {
    @InjectView(R.id.TextView_PuzzleName)
    TextView puzzleNameView;
    @InjectView(R.id.TextView_Category)
    TextView categoryView;
    @InjectView(R.id.TextView_Published)
    TextView publishedView;
    @InjectView(R.id.TableLayout_Authors)
    TableLayout authorsLayoutView;
    @InjectView(R.id.TextView_Number_Questions)
    TextView numberOfQuestionsView;
    @InjectView(R.id.Button_Play)
    Button playButton;
    @InjectView(R.id.TextView_Rating)
    TextView ratingView;
    @InjectView(R.id.TextView_TimesPlayed)
    TextView timesPlayedView;

    private ContentHelper helper;
    private boolean receiverRegistered = false;
    private int puzzleId = 0;
    public static final int GET_PUZZLE_TO_PLAY = 8;
    private ArrayList<ContentValues> questionSets;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_play);
        helper = new ContentHelper(getBaseContext());

        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            final int idInt = extras.containsKey(General.ID) ? extras.getInt(General.ID) : 0;
            if (idInt != 0) {
                puzzleId = idInt;
                startUpdates();

                playButton.setEnabled(false);
                playButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Intent intent = new Intent(getBaseContext(), PlayActivity.class);
                        intent.putExtra(General.ID, puzzleId);
                        intent.putParcelableArrayListExtra(General.QUESTIONSET, questionSets);
                        startActivity(intent);
                    }
                });
            }
        } else {
            startActivity(new Intent(getBaseContext(), MainActivity.class));
            Toast.makeText(getBaseContext(), R.string.error, Toast.LENGTH_SHORT).show();
        }
    }

    private void fillAuthorTable() {
        ArrayList<String> nicknames = helper.getAuthors(puzzleId);
        for (String nickname : nicknames) {
            insertRow(nickname);
            authorsLayoutView.setVisibility(View.VISIBLE);
        }
    }

    private void insertRow(String name) {
        final TableRow newRow = new TableRow(MainPlayActivity.this);
        TextView textView = new TextView(this);
        textView.setText(name);
        textView.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                TextView text = (TextView) view;
                ContentValues cv = new ContentValues();
                cv.put(General.XML_TAG_NICKNAME, text.getText().toString());
                helper.addFriend(cv);
                Toast.makeText(getBaseContext(), R.string.friend_added, Toast.LENGTH_SHORT).show();
            }
        });
        textView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 18);
        newRow.addView(textView);
        authorsLayoutView.addView(newRow);
    }

    private void startUpdates() {
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UpdaterService.GET_PUZZLE_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        receiverRegistered = true;

        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
        intent.putExtra(General.ID, puzzleId);
        intent.putExtra(General.UPDATER_ACTION, GET_PUZZLE_TO_PLAY);
        startService(intent);
    }

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case GET_PUZZLE_TO_PLAY:
                    ContentValues puzzle = helper.getPuzzle(puzzleId);
                    questionSets = helper.getQuestionSet(puzzleId);

                    puzzleNameView.setText(puzzle.getAsString(General.PUZZLE_NAME));
                    publishedView.setText(puzzle.getAsString(General.XML_TAG_PUBLISHDATE));
                    categoryView.setText(puzzle.getAsString(General.PUZZLE_CATEGORY));
                    timesPlayedView.setText(puzzle.getAsString(General.XML_TAG_TIMESPLAYED));
                    ratingView.setText(puzzle.getAsString(General.XML_TAG_RATING));
                    numberOfQuestionsView.setText("" + questionSets.size());
                    fillAuthorTable();
                    playButton.setEnabled(true);
                    break;
            }
        }
    };

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // Handle the back button on MainActivity
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            finish();
            return super.onKeyDown(keyCode, event);
        } else {
            return super.onKeyDown(keyCode, event);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }
}
