package com.master.playit;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.*;
import com.master.playit.helper.General;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;
import java.util.Collections;
import java.util.TimerTask;

public class PlayActivity extends RoboActivity implements View.OnClickListener {

    @InjectView(R.id.TextView_Play_Title)
    TextView titleView;
    @InjectView(R.id.TextView_TimeLabel)
    TextView timeLabel;
    @InjectView(R.id.TextView_Question)
    TextView question;
    @InjectView(R.id.ProgressBar_Play)
    ProgressBar progressbar;
    @InjectView(R.id.LinearLayout_Written)
    LinearLayout writtenLayout;
    @InjectView(R.id.LinearLayout_Mpc)
    LinearLayout mpcLayout;
    @InjectView(R.id.EditText_Written)
    EditText writtenText;
    @InjectView(R.id.LinearLayout_Spinner)
    LinearLayout spinnerLayout;
    @InjectView(R.id.Spinner_DropDown)
    Spinner spinner;
    @InjectView(R.id.Button_Hint)
    Button hintButton;

    @InjectView(R.id.TextView_Answer_1)
    TextView answer1;
    @InjectView(R.id.TextView_Answer_2)
    TextView answer2;
    @InjectView(R.id.TextView_Answer_3)
    TextView answer3;
    @InjectView(R.id.TextView_Answer_4)
    TextView answer4;
    @InjectView(R.id.TextView_Answer_5)
    TextView answer5;

    private int puzzleId;
    private ArrayList<ContentValues> questionSet;
    private Handler handler = new Handler();
    private java.util.Timer timer = new java.util.Timer();
    private int currentQuestion;
    private int currentInputMethod;
    private long startTime;
    private SharedPreferences settings;
    private ArrayList<String> alternatives = new ArrayList<String>(8);

    // Runs without timer be re posting self
    private Runnable run = new Runnable() {
        @Override
        public void run() {
            long millis = System.currentTimeMillis() - startTime;
            int seconds = (int) (millis / 1000);
            int minutes = seconds / 60;
            seconds = seconds % 60;
            timeLabel.setText(String.format("%d:%02d", minutes, seconds));
            handler.postDelayed(this, 500);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.play);
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);

        long timeSpent = 0;
        currentQuestion = settings.contains(General.GAME_PREFERENCES_CURRENT_QUESTION) ? settings.getInt(General.GAME_PREFERENCES_CURRENT_QUESTION, 0) : 0;
        startTime = System.currentTimeMillis() - timeSpent;

        Bundle extras = getIntent().getExtras();
        initPlay(extras);
        updateScreen();

        if (currentQuestion == 0) {
            timer = new java.util.Timer();
            timer.schedule(new Timer(), 0, 500);
            handler.postDelayed(run, 0);
        } else if (currentQuestion == questionSet.size() - 1) {
            timer.cancel();
            timer.purge();
            handler.removeCallbacks(run);
        }
        if (currentInputMethod != R.string.dropdown) {
            hintButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    revealHint();
                }
            });
            hintButton.setVisibility(View.VISIBLE);
        } else {
            hintButton.setVisibility(View.INVISIBLE);
        }
    }

    private void revealHint() {
        String answer = questionSet.get(currentQuestion).getAsString(General.XML_TAG_ANSWER).toLowerCase();
        ArrayList<TextView> textViews = new ArrayList<TextView>();
        textViews.add(answer1);
        textViews.add(answer2);
        textViews.add(answer3);
        textViews.add(answer4);
        textViews.add(answer5);
        switch (currentInputMethod) {
            case R.string.written:
                writtenText.setText(answer.substring(0, 1));
                hintButton.setVisibility(View.INVISIBLE);
                break;
            case R.string.mpcWords:
                Collections.shuffle(textViews);
                for (TextView view : textViews) {
                    if (view.getText().toString() != answer) {
                        view.setVisibility(View.GONE);
                        hintButton.setVisibility(View.INVISIBLE);
                        break;
                    }
                }
                break;
            case R.string.mpc:
                Collections.shuffle(textViews);
                for (TextView view : textViews) {
                    if (view.getText().toString() != answer) {
                        view.setVisibility(View.GONE);
                        hintButton.setVisibility(View.INVISIBLE);
                        break;
                    }
                }
                break;
            case R.string.dropdown:
                break;
        }

    }

    private void addMpcText() {
        ArrayList<TextView> textViews = new ArrayList<TextView>();
        textViews.add(answer1);
        textViews.add(answer2);
        textViews.add(answer3);
        textViews.add(answer4);
        textViews.add(answer5);
        Collections.shuffle(textViews);
        if (alternatives.size() > textViews.size()) {
            for (int i = 0; i < textViews.size(); i++) {
                textViews.get(i).setText(alternatives.get(i).toString());
                textViews.get(i).setVisibility(View.VISIBLE);
            }
        } else {
            for (int i = 0; i < alternatives.size(); i++) {
                textViews.get(i).setText(alternatives.get(i).toString());
                textViews.get(i).setVisibility(View.VISIBLE);
            }
        }
    }

    private void generateShuffledAlternatives() {
        alternatives.clear();
        String answer = questionSet.get(currentQuestion).getAsString(General.XML_TAG_ANSWER);
        char[] characters = answer.toLowerCase().toCharArray();
        ArrayList<Character> answers = new ArrayList<Character>();
        for (int i = 0; i < characters.length - 1; i++) {
            // Add characters to an ArrayList, one character in each position
            answers.add(characters[i]);
        }
        alternatives.add(0, answer.toLowerCase());
        for (int i = 0; i < 8; i++) {
            Collections.shuffle(answers);
            StringBuilder builder = new StringBuilder(answers.size());
            for (Character ch : answers) {
                builder.append(ch);
            }
            alternatives.add(builder.toString());
        }
    }

    private void generateWordAlternatives() {
        alternatives.clear();
        String answer = questionSet.get(currentQuestion).getAsString(General.XML_TAG_ANSWER);
        for (ContentValues cv : questionSet) {
            if (!cv.getAsString(General.XML_TAG_ANSWER).equals(answer)) {
                alternatives.add(cv.getAsString(General.XML_TAG_ANSWER));
            }
        }
        Collections.shuffle(alternatives);
        alternatives.add(0, answer);
    }

    private void initPlay(Bundle extras) {
        if (extras != null) {
            final int id = extras.containsKey(General.ID) ? extras.getInt(General.ID) : 0;
            if (id != 0) {
                puzzleId = id;
                questionSet = extras.getParcelableArrayList(General.QUESTIONSET);
                currentQuestion = settings.contains(General.GAME_PREFERENCES_CURRENT_QUESTION) ? settings.getInt(General.GAME_PREFERENCES_CURRENT_QUESTION, 0) : 0;
                progressbar.setMax(questionSet.size());
            }
        }
    }

    // Tells activity to run on ui thread
    class Timer extends TimerTask {
        @Override
        public void run() {
            PlayActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    long millis = System.currentTimeMillis() - startTime;
                    int seconds = (int) (millis / 1000);
                    int minutes = seconds / 60;
                    seconds = seconds % 60;
                    timeLabel.setText(String.format("%d:%02d", minutes, seconds));
                }
            });
        }
    }

    public static String lcs(String a, String b) {
        String x, y;
        int alen = a.length();
        int blen = b.length();
        if (alen == 0 || blen == 0) {
            return "";
        } else if (a.charAt(alen - 1) == b.charAt(blen - 1)) {
            return lcs(a.substring(0, alen - 1), b.substring(0, blen - 1)) + a.charAt(alen - 1);
        } else {
            x = lcs(a, b.substring(0, blen - 1));
            y = lcs(a.substring(0, alen - 1), b);
        }
        return (x.length() > y.length()) ? x : y;
    }

    public static int startEvaluator(String answered, String answer) {
        int answeredLength = answered.length();
        int answerLength = answer.length();
        int score = 0;
        if (answeredLength >= answerLength) {
            for (int i = 0; i < answerLength; i++) {
                while (answered.charAt(i) == answer.charAt(i)) {
                    score++;
                }
            }
        } else {
            for (int i = 0; i < answerLength; i++) {
                while (answered.charAt(i) == answer.charAt(i)) {
                    score++;
                }
            }
        }
        return score;
    }

    @Override
    public void onClick(View view) {
        TextView textView = (TextView) view;
        float score;
        String answered = textView.getText().toString();
        if (answered.length() > 0) {
            questionSet.get(currentQuestion).put(General.ANSWERED, answered);
            answered = answered.toLowerCase();
            String answer = questionSet.get(currentQuestion).getAsString(General.XML_TAG_ANSWER).toLowerCase();
            if (answered.equals(answer)) {
                question.setTextColor(getResources().getColor(R.color.green));
                if (hintButton.getVisibility() == View.INVISIBLE && currentInputMethod != R.string.dropdown) {
                    score = 8;
                    questionSet.get(currentQuestion).put(General.HINT, true);
                } else score = 10;
                // Answer is correct
            } else {
                question.setTextColor(getResources().getColor(R.color.orange));
                // Calculate correctness of answer; Longest Common subSequence between 1 and 10 based on lcs/answer
                String correctness = lcs(answer, answered);
                score = ((float) (correctness.length() * 10) / (float) (answer.length() * 10)) * 10;
                // int start = startEvaluator(answered, answer);
                // if (start < score) score = score / 2;
                if (hintButton.getVisibility() == View.INVISIBLE && currentInputMethod != R.string.dropdown) {
                    score = score * 4 / 5;
                    questionSet.get(currentQuestion).put(General.HINT, true);
                }
            }
            questionSet.get(currentQuestion).put(General.SCORE, (int) Math.ceil(score));
            currentQuestion++;
            hideAllInputs();
            updateScreen();
        }
    }

    private void updateScreen() {
        if (currentQuestion == questionSet.size()) {
            Intent intent = new Intent(getBaseContext(), ResultActivity.class);
            intent.putExtra(General.QUESTIONSET, questionSet);
            intent.putExtra(General.ID, puzzleId);
            intent.putExtra(General.TIME, timeLabel.getText().toString());
            startActivity(intent);
            finish();
        } else {
            hintButton.setVisibility(View.VISIBLE);
            titleView.setText("#" + (currentQuestion + 1));
            progressbar.setProgress(currentQuestion + 1);
            question.setText(questionSet.get(currentQuestion).getAsString(General.XML_TAG_QUESTION));
            generateShuffledAlternatives();
            String inputMethod = questionSet.get(currentQuestion).getAsString(General.XML_TAG_INPUTMETHOD);
            if (inputMethod.equals(getResources().getString(R.string.written))) {
                currentInputMethod = R.string.written;
                writtenLayout.setVisibility(View.VISIBLE);
                writtenText.setText("");
                //Forcing keyboard
                writtenText.requestFocus();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.showSoftInput(writtenText, InputMethodManager.SHOW_IMPLICIT);
                writtenText.setOnKeyListener(new View.OnKeyListener() {
                    @Override
                    public boolean onKey(View view, int keyCode, KeyEvent keyEvent) {
                        if (keyCode == KeyEvent.KEYCODE_ENTER && keyEvent.getAction() == KeyEvent.ACTION_DOWN) {
                            onClick(view);
                            return true;
                        }
                        return false;
                    }
                });
            } else if (inputMethod.equals(getResources().getString(R.string.mpc))) {
                currentInputMethod = R.string.mpc;
                mpcLayout.setVisibility(View.VISIBLE);
                addMpcText();
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.showSoftInput(writtenText, InputMethodManager.HIDE_NOT_ALWAYS);
            } else if (inputMethod.equals(getResources().getString(R.string.mpcWords))) {
                currentInputMethod = R.string.mpcWords;
                generateWordAlternatives();
                mpcLayout.setVisibility(View.VISIBLE);
                addMpcText();
            } else if (inputMethod.equals(getResources().getString(R.string.dropdown))) {
                hintButton.setVisibility(View.INVISIBLE);
                currentInputMethod = R.string.dropdown;
                spinnerLayout.setVisibility(View.VISIBLE);
                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_dropdown_item, alternatives);
                Collections.shuffle(alternatives);
                alternatives.add(0, "");
                spinner.setAdapter(adapter);
                spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                    @Override
                    public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                        onClick(view);
                    }

                    @Override
                    public void onNothingSelected(AdapterView<?> adapterView) {

                    }
                });
            }
            question.setTextColor(getResources().getColor(R.color.yellow));
        }
    }

    private void hideAllInputs() {
        writtenLayout.setVisibility(View.GONE);
        mpcLayout.setVisibility(View.GONE);
        spinnerLayout.setVisibility(View.GONE);
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
    }

    @Override
    protected void onStart() {
        super.onStart();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
