package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.*;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.provider.Settings;
import android.view.*;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class QuestionSetActivity extends RoboActivity implements GestureDetector.OnGestureListener {
    @InjectView(R.id.EditText_Question)
    EditText editQuestion;
    @InjectView(R.id.EditText_Answer)
    EditText editAnswer;
    @InjectView(R.id.TextView_QuestionSet_Title)
    TextView title;
    @InjectView(R.id.Spinner_InputMethod)
    Spinner spinner;
    @InjectView(R.id.ProgressBar_QuestionSet)
    ProgressBar progressBar;

    private static int SWIPE_MIN_DISTANCE;
    private static final int SWIPE_MAX_OFF_PATH = 250;
    private static int SWIPE_THRESHOLD_VELOCITY;

    public static final int QUESTION_SET_UPDATE = 4;
    private static final int NEED_INTERNET_DIALOG = 5;
    private static final int PROGRESS_DIALOG = 6;

    private SharedPreferences settings;
    private ArrayList<ContentValues> oldQuestionSet;
    private ArrayList<ContentValues> newQuestionSet;
    private GestureDetector gestureDetector;
    private ContentHelper helper;
    private boolean receiverRegistered = false;
    private boolean collaborating = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.create_questionset);
        helper = new ContentHelper(this);

        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UpdaterService.COLLABORATE_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        receiverRegistered = true;

        final ViewConfiguration vc = ViewConfiguration.get(this);
        SWIPE_MIN_DISTANCE = vc.getScaledTouchSlop();
        SWIPE_THRESHOLD_VELOCITY = vc.getScaledMinimumFlingVelocity();

        // Gesture detection
        gestureDetector = new GestureDetector(this);

        // Init private fields
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        collaborating = getIntent().getBooleanExtra(General.COLLABORATE, false);
        oldQuestionSet = getIntent().getParcelableArrayListExtra(General.QUESTIONSET);
        newQuestionSet = getIntent().getParcelableArrayListExtra(General.NEW_QUESTIONSET);
        if (oldQuestionSet == null) {
            Toast.makeText(getBaseContext(), R.string.swipe_questionset, Toast.LENGTH_SHORT).show();
            oldQuestionSet = new ArrayList<ContentValues>();
            // Put last questionSet in new as current (first in old)
            if (newQuestionSet.size() > 0) {
                oldQuestionSet.add(newQuestionSet.get(newQuestionSet.size() - 1));
                newQuestionSet.remove(newQuestionSet.get(newQuestionSet.size() - 1));
            }
        }

        // ProgressBar
        progressBar.setProgress(newQuestionSet.size() + 1);
        if (oldQuestionSet.size() + newQuestionSet.size() < 10) {
            progressBar.setMax(10);
        } else {
            progressBar.setMax(20);
        }
        if (newQuestionSet.size() == 10) {
            Toast.makeText(getBaseContext(), R.string.minimum_questionset, Toast.LENGTH_SHORT).show();
        }

        // Init field values
        initSpinner();
        title.setText("Question Set #" + (newQuestionSet.size() + 1));
        if (oldQuestionSet.size() > 0) {
            editQuestion.setText(oldQuestionSet.get(0).getAsString(General.XML_TAG_QUESTION));
            editAnswer.setText(oldQuestionSet.get(0).getAsString(General.XML_TAG_ANSWER));
        }
    }

    private void nextEventAction() {
        if (!saveCurrentSet()) {
            Toast.makeText(getBaseContext(), R.string.fill_question, Toast.LENGTH_SHORT).show();
        } else {
            Intent intent = new Intent(getBaseContext(), QuestionSetActivity.class);
            intent.putParcelableArrayListExtra(General.QUESTIONSET, oldQuestionSet);
            intent.putParcelableArrayListExtra(General.NEW_QUESTIONSET, newQuestionSet);
            startActivity(intent);
        }
    }

    private boolean saveCurrentSet() {
        String question = editQuestion.getText().toString();
        String answer = editAnswer.getText().toString();
        if (!question.equals("") && !answer.equals("")) {
            if (oldQuestionSet.size() > 0) {
                String oldQuestion = oldQuestionSet.get(0).getAsString(General.XML_TAG_QUESTION);
                String oldAnswer = oldQuestionSet.get(0).getAsString(General.XML_TAG_ANSWER);
                String spinnerText = oldQuestionSet.get(0).getAsString(General.XML_TAG_INPUTMETHOD);
                if (question.equals(oldQuestion) && answer.equals(oldAnswer) && spinner.getSelectedItem().toString().equals(spinnerText)) {
                    newQuestionSet.add(oldQuestionSet.get(0));
                    oldQuestionSet.remove(0);
                    return true;
                }
            }
            ContentValues cv = new ContentValues();
            cv.put(General.XML_TAG_QUESTION, question);
            cv.put(General.XML_TAG_ANSWER, answer);
            cv.put(General.XML_TAG_INPUTMETHOD, spinner.getSelectedItem().toString());
            newQuestionSet.add(cv);
            if (oldQuestionSet.size() != 0)
                oldQuestionSet.remove(0);
            return true;
        }
        return false;
    }

    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // Handle the back button from MainCreate to MainActivity
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            previousEventAction();
            return true;
        } else if (keyCode == KeyEvent.KEYCODE_HOME) {
            if (saveCurrentSet()) {
                newQuestionSet.addAll(oldQuestionSet);
                helper.saveQuestionSet(settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0), newQuestionSet);
                Toast.makeText(getBaseContext(), R.string.saved, Toast.LENGTH_SHORT).show();
            }
            return super.onKeyDown(keyCode, event);
        } else {
            return super.onKeyDown(keyCode, event);
        }
    }

    private void previousEventAction() {
        if (newQuestionSet.size() == 0) {
            // At the start of the questionSets, save and return to CreateActivity
            if (!editQuestion.getText().toString().equals("")) {
                saveCurrentSet();
                oldQuestionSet.add(0, newQuestionSet.get(0));
                newQuestionSet.remove(newQuestionSet.get(0));
            }
            if (oldQuestionSet.size() > 0) {
                helper.saveQuestionSet(settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0), oldQuestionSet);
                Toast.makeText(getBaseContext(), R.string.saved, Toast.LENGTH_SHORT).show();
            }
            Intent intent = new Intent(getBaseContext(), CreateActivity.class);
            int puzzleId = settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0);
            intent.putExtra(General.ID, puzzleId);
            startActivity(intent);
            finish();
        } else {
            if (saveCurrentSet()) {
                // Switching the last questionSets in the newSet to first in the oldSet
                oldQuestionSet.add(0, newQuestionSet.get(newQuestionSet.size() - 1));
                newQuestionSet.remove(newQuestionSet.get(newQuestionSet.size() - 1));
                Toast.makeText(getBaseContext(), R.string.questionSet_saved, Toast.LENGTH_SHORT).show();
            }
            // Switching the last questionSets in the newSet to first in the oldSet
            oldQuestionSet.add(0, newQuestionSet.get(newQuestionSet.size() - 1));
            newQuestionSet.remove(newQuestionSet.get(newQuestionSet.size() - 1));

            Intent intent = new Intent(getBaseContext(), QuestionSetActivity.class);
            intent.putParcelableArrayListExtra(General.QUESTIONSET, oldQuestionSet);
            intent.putParcelableArrayListExtra(General.NEW_QUESTIONSET, newQuestionSet);
            startActivity(intent);
        }
    }

    @Override
    public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
        try {
            if (Math.abs(e1.getY() - e2.getY()) > SWIPE_MAX_OFF_PATH) return false;
            // right to left swipe
            if (e1.getX() - e2.getX() > SWIPE_MIN_DISTANCE && Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY) {
                nextEventAction();
                // Toast.makeText(QuestionSetActivity.this, "Left Swipe", Toast.LENGTH_SHORT).show();
            } else if (e2.getX() - e1.getX() > SWIPE_MIN_DISTANCE && Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY) {
                previousEventAction();
                // Toast.makeText(QuestionSetActivity.this, "Right Swipe", Toast.LENGTH_SHORT).show();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case CreateActivity.COLLABORATE_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.collaborated, Toast.LENGTH_SHORT).show();
                    break;
                case QuestionSetActivity.QUESTION_SET_UPDATE:
                    ArrayList<ContentValues> questionSet = helper.getQuestionSet(settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0));
                    Intent i = new Intent(getBaseContext(), QuestionSetActivity.class);
                    i.putParcelableArrayListExtra(General.NEW_QUESTIONSET, questionSet);
                    startActivity(i);
                    dismissDialog(PROGRESS_DIALOG);
                    break;
            }
        }
    };

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        super.onOptionsItemSelected(item);
        Intent intent = item.getIntent();
        int puzzleId = settings.getInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, 0);
        if (item.getItemId() == R.id.back_menu_item) {
            saveCurrentSet();
            newQuestionSet.addAll(oldQuestionSet);
            helper.saveQuestionSet(puzzleId, newQuestionSet);
            intent.putExtra(General.ID, puzzleId);
            startActivity(intent);
        } else if (item.getItemId() == R.id.update_menu_item) {
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.QUESTION_SET_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            intent.putExtra(General.UPDATER_ACTION, QUESTION_SET_UPDATE);
            intent.putExtra(General.ID, puzzleId);
            ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
            // ARE WE CONNECTED TO THE INTERNET?
            if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                    && conMgr.getActiveNetworkInfo().isConnected())) {
                showDialog(NEED_INTERNET_DIALOG);
            } else {
                receiverRegistered = true;
                showDialog(PROGRESS_DIALOG);
                startService(intent);
            }
        } else {
            // Delete question
            oldQuestionSet.remove(0);
            intent.putParcelableArrayListExtra(General.QUESTIONSET, oldQuestionSet);
            intent.putParcelableArrayListExtra(General.NEW_QUESTIONSET, newQuestionSet);
            startActivity(intent);
        }
        finish();
        return true;
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
            case NEED_INTERNET_DIALOG:
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setMessage(R.string.search_need_internet);
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                        startActivity(new Intent(getBaseContext(), MainActivity.class));
                    }
                });
                builder.setPositiveButton(R.string.to_settings, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
                    }
                });
                return builder.create();
            case PROGRESS_DIALOG:
                ProgressDialog dialog = new ProgressDialog(QuestionSetActivity.this, ProgressDialog.STYLE_SPINNER);
                dialog.setIndeterminate(true);
                dialog.setMessage(getResources().getString(R.string.updating));
                dialog.setCancelable(false);
                return dialog;
        }
        return null;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        if (collaborating) {
            getMenuInflater().inflate(R.menu.collaborate_options, menu);
        } else {
            getMenuInflater().inflate(R.menu.question_options, menu);
        }
        menu.findItem(R.id.back_menu_item).setIntent(new Intent(this, CreateActivity.class));
        menu.findItem(R.id.delete_menu_item).setIntent(new Intent(this, QuestionSetActivity.class));
        if (collaborating)
            menu.findItem(R.id.update_menu_item).setIntent(new Intent(this, UpdaterService.class));
        return true;
    }

    private void initSpinner() {
        ArrayAdapter<?> adapter = ArrayAdapter.createFromResource(this, R.array.inputmethod, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);
        if (oldQuestionSet.size() > 0) {
            String inputMethod = oldQuestionSet.get(0).getAsString(General.XML_TAG_INPUTMETHOD);
            if (inputMethod.equals(getResources().getString(R.string.mpc))) {
                spinner.setSelection(0);
            } else if (inputMethod.equals(getResources().getString(R.string.mpcWords))) {
                spinner.setSelection(1);
            } else if (inputMethod.equals(getResources().getString(R.string.dropdown))) {
                spinner.setSelection(2);
            } else if (inputMethod.equals(getResources().getString(R.string.rearranged))) {
                spinner.setSelection(3);
            } else if (inputMethod.equals(getResources().getString(R.string.grid))) {
                spinner.setSelection(4);
            } else if (inputMethod.equals(getResources().getString(R.string.written))) {
                spinner.setSelection(5);
            }
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent me) {
        return gestureDetector.onTouchEvent(me);
    }

    @Override
    public boolean onDown(MotionEvent motionEvent) {
        return false;
    }

    @Override
    public void onShowPress(MotionEvent motionEvent) {

    }

    @Override
    public boolean onSingleTapUp(MotionEvent motionEvent) {
        return false;
    }

    @Override
    public boolean onScroll(MotionEvent motionEvent, MotionEvent motionEvent2, float v, float v2) {
        return false;
    }

    @Override
    public void onLongPress(MotionEvent motionEvent) {

    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }
}
