package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.*;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.view.View;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.ResultAdapter;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class ResultActivity extends RoboActivity {

    @InjectView(R.id.ListView_Results)
    ListView resultLayout;
    @InjectView(R.id.TextView_Result_Time)
    TextView timeResult;
    @InjectView(R.id.TextView_Result_Score)
    TextView scoreResult;
    @InjectView(R.id.Button_Rate)
    Button rateButton;
    @InjectView(R.id.Button_Scores)
    Button viewScoresButton;

    private ResultAdapter adapter;
    private ArrayList<ContentValues> questionSet;
    private SharedPreferences settings;
    private ContentHelper helper;
    private int puzzleId;
    private boolean receiverRegistered = false;

    private static final int RATE_DIALOG = 1;
    public static final int RATE_PUZZLE = 11;
    public static final int UPLOAD_SCORE = 15;
    public static final int ADD_TIMES_PLAYED = 16;

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case RATE_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.puzzle_rated, Toast.LENGTH_SHORT).show();
                    break;
                case UPLOAD_SCORE:

                    break;
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.result);
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        helper = new ContentHelper(getBaseContext());

        final Bundle extras = getIntent().getExtras();
        questionSet = (ArrayList<ContentValues>) extras.getSerializable(General.QUESTIONSET);
        puzzleId = extras.getInt(General.ID);
        String timeLabel = extras.getString(General.TIME);

        int score = 0;
        for (ContentValues cv : questionSet)
            score += cv.getAsInteger(General.SCORE);
        final int totalScore = score;

        timeResult.setText("Time: " + timeLabel);
        scoreResult.setText("Score: " + score);

        adapter = new ResultAdapter(this, questionSet);
        resultLayout.setAdapter(adapter);

        SharedPreferences.Editor editor = settings.edit();
        editor.putInt(General.GAME_PREFERENCES_CURRENT_QUESTION, 0);
        editor.commit();

        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        final boolean notConnectedToTheInternet = !(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable() && conMgr.getActiveNetworkInfo().isConnected());

        if (!helper.isPuzzleRated(puzzleId)) {
            rateButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    if (notConnectedToTheInternet) {
                        Toast.makeText(getBaseContext(), R.string.rate_need_internet, Toast.LENGTH_SHORT).show();
                    } else {
                        showDialog(RATE_DIALOG);
                    }
                }
            });
            rateButton.setVisibility(View.VISIBLE);
        } else {
            rateButton.setVisibility(View.INVISIBLE);
            helper.saveLatestPlayedPuzzle(puzzleId, null, null);
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.ADD_TIMES_PLAYED_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            receiverRegistered = true;

            Intent intent = new Intent(getBaseContext(), UpdaterService.class);
            intent.putExtra(General.UPDATER_ACTION, ADD_TIMES_PLAYED);
            startService(intent);
        }

        viewScoresButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (notConnectedToTheInternet) {
                    Toast.makeText(getBaseContext(), R.string.scores_need_internet, Toast.LENGTH_SHORT).show();
                } else {
                    Intent intent = new Intent(getBaseContext(), ScoreActivity.class);
                    intent.putExtra(General.SCORE, totalScore);
                    intent.putExtras(extras);
                    startActivity(intent);
                }
            }
        });
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UpdaterService.UPLOAD_SCORES_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        intentFilter.addAction(UpdaterService.ADD_TIMES_PLAYED_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        receiverRegistered = true;

        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
        intent.putExtra(General.ID, puzzleId);
        intent.putExtra(General.SCORE, score);
        intent.putExtra(General.TIME, timeLabel);
        intent.putExtra(General.UPDATER_ACTION, UPLOAD_SCORE);
        startService(intent);

    }

    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
            case RATE_DIALOG:
                AlertDialog.Builder builder = new AlertDialog.Builder(ResultActivity.this);
                final NumberPicker picker = new NumberPicker(ResultActivity.this);
                picker.setMaxValue(10);
                picker.setMinValue(0);
                builder.setTitle(R.string.rate_puzzle);
                builder.setView(picker);
                builder.setPositiveButton(R.string.rate, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        IntentFilter intentFilter = new IntentFilter();
                        intentFilter.addAction(UpdaterService.PUZZLE_RATED_DONE);
                        registerReceiver(uploadReceiver, intentFilter);
                        receiverRegistered = true;
                        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                        intent.putExtra(General.ID, puzzleId);
                        intent.putExtra(General.PUZZLE_RATINGGIVEN, picker.getValue());
                        intent.putExtra(General.UPDATER_ACTION, RATE_PUZZLE);
                        startService(intent);
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                return builder.create();
        }
        return null;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }
}
