package com.master.playit;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.*;
import android.os.Bundle;
import android.widget.TabHost;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class ScoreActivity extends RoboActivity {
    @InjectView(R.id.TabHost1)
    TabHost host;
    @InjectView(R.id.TableLayout_AllScores)
    TableLayout allScoresLayout;
    @InjectView(R.id.TableLayout_FriendScores)
    TableLayout friendScoresLayout;

    private SharedPreferences settings;
    private ContentHelper helper;
    private boolean receiverRegistered = false;
    public static final int FETCH_SCORES = 14;

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case FETCH_SCORES:
                    ArrayList<ContentValues> scores = intent.getParcelableArrayListExtra(General.SCORES);
                    processScores(scores);
                    dismissDialog(FETCH_SCORES);
                    break;
            }

        }
    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.scores);
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        helper = new ContentHelper(getBaseContext());

        initTabs();
        initializeHeaderRow(allScoresLayout);
        initializeHeaderRow(friendScoresLayout);

        Bundle extras = getIntent().getExtras(); // PuzzleId
        int puzzleId = extras.getInt(General.ID);

        showDialog(FETCH_SCORES);
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(UpdaterService.FETCH_SCORES_DONE);
        registerReceiver(uploadReceiver, intentFilter);
        receiverRegistered = true;

        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
        intent.putExtra(General.ID, puzzleId);
        intent.putExtra(General.UPDATER_ACTION, FETCH_SCORES);
        startService(intent);
    }

    private void initTabs() {
        host.setup();
        TabHost.TabSpec allScoresTab = host.newTabSpec("allTab");
        allScoresTab.setIndicator(getResources().getString(R.string.all_scores), getResources().getDrawable(android.R.drawable.star_on));
        allScoresTab.setContent(R.id.ScrollViewAllScores);
        host.addTab(allScoresTab);

        TabHost.TabSpec friendScoresTab = host.newTabSpec("friendTab");
        friendScoresTab.setIndicator(getResources().getString(R.string.friend_scores), getResources().getDrawable(android.R.drawable.star_on));
        friendScoresTab.setContent(R.id.ScrollViewFriendScores);
        host.addTab(friendScoresTab);
        host.setCurrentTabByTag("allTab");
    }

    private void initializeHeaderRow(TableLayout scoreTable) {
        // Create the Table header row
        TableRow headerRow = new TableRow(this);
        int textColor = getResources().getColor(R.color.app_text_color);
        float textSize = getResources().getDimension(R.dimen.thirteendp);
        addTextToRowWithValues(headerRow, getResources().getString(R.string.rank), textColor, textSize);
        addTextToRowWithValues(headerRow, getResources().getString(R.string.nickname), textColor, textSize);
        addTextToRowWithValues(headerRow, getResources().getString(R.string.score), textColor, textSize);
        addTextToRowWithValues(headerRow, getResources().getString(R.string.time), textColor, textSize);
        scoreTable.addView(headerRow);
    }

    private void processScores(ArrayList<ContentValues> scores) {
        ArrayList<String> friends = helper.getFriends();
        friends.add(settings.getString(General.GAME_PREFERENCES_NICKNAME, ""));
        for (int i = 0; i < scores.size(); i++) {
            insertScoreRow(allScoresLayout, i+1, scores.get(i));
            for (String friend : friends) {
                if (friend.equals(scores.get(i).getAsString(General.XML_TAG_NICKNAME))) {
                    insertScoreRow(friendScoresLayout, i + 1, scores.get(i));
                }
            }
        }
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        if(id == FETCH_SCORES){
            ProgressDialog dialog = new ProgressDialog(ScoreActivity.this, ProgressDialog.STYLE_SPINNER);
            dialog.setIndeterminate(true);
            dialog.setMessage(getResources().getString(R.string.updating));
            dialog.setCancelable(false);
            return dialog;
        }
        return null;
    }

    private void insertScoreRow(final TableLayout scoreTable, int rank, ContentValues score) {
        final TableRow newRow = new TableRow(ScoreActivity.this);
        int textColor = getResources().getColor(R.color.app_text_color);
        float textSize = getResources().getDimension(R.dimen.tendp);

        addTextToRowWithValues(newRow, String.valueOf(rank), textColor, textSize);
        addTextToRowWithValues(newRow, score.getAsString(General.XML_TAG_NICKNAME), textColor, textSize);
        addTextToRowWithValues(newRow, String.valueOf(score.getAsInteger(General.SCORE)), textColor, textSize);
        addTextToRowWithValues(newRow, score.getAsString(General.TIME), textColor, textSize);
        scoreTable.addView(newRow);
    }

    private void addTextToRowWithValues(final TableRow tableRow, String text, int textColor, float textSize) {
        TextView textView = new TextView(this);
        textView.setPadding(5, 5, 5, 0);
        textView.setTextSize(textSize);
        textView.setTextColor(textColor);
        textView.setText(text);
        tableRow.addView(textView);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }
}
