package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.*;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.provider.Settings;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.*;
import com.master.playit.helper.General;
import com.master.playit.helper.LazyAdapter;
import com.master.playit.helper.UpdaterService;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * User: Malin
 */
public class SearchActivity extends RoboActivity {

    @InjectView(R.id.ListView_Search)
    ListView searchView;
    @InjectView(R.id.Spinner_SearchInput)
    Spinner spinner;

    private static final int NEED_INTERNET_DIALOG = 1;
    public static final int GET_SEARCH_RESULTS = 10;

    private LazyAdapter adapter;
    private SharedPreferences settings;
    private boolean receiverRegistered = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.search);

        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        checkNetwork();

        List<String> categories = new ArrayList<String>();
        Collections.addAll(categories, getResources().getStringArray(R.array.categories));
        categories.add(0, "");
        ArrayAdapter<String> spinnerAdapter = new ArrayAdapter<String>(getBaseContext(), android.R.layout.simple_spinner_dropdown_item, categories);
        spinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(spinnerAdapter);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                String searchText = spinner.getSelectedItem().toString();
                if (!searchText.equals("")) {
                    IntentFilter intentFilter = new IntentFilter();
                    intentFilter.addAction(UpdaterService.GET_SEARCH_RESULTS_DONE);
                    registerReceiver(downloadReceiver, intentFilter);
                    receiverRegistered = true;

                    Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                    intent.putExtra(General.PUZZLE_CATEGORY, searchText);
                    intent.putExtra(General.UPDATER_ACTION, GET_SEARCH_RESULTS);
                    startService(intent);
                }
            }
            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });
        spinner.setSelection(0);

        searchView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                Intent intent = new Intent(getBaseContext(), MainPlayActivity.class);
                Integer puzzleId = Integer.parseInt(view.getTag(R.id.ID_TAG).toString());
                intent.putExtra(General.ID, puzzleId);
                startActivity(intent);
            }
        });
    }

    private BroadcastReceiver downloadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case GET_SEARCH_RESULTS:
                    ArrayList<ContentValues> puzzles = intent.getParcelableArrayListExtra(General.SEARCH_RESULTS);
                    if (searchView.getChildCount() > 0)
                        searchView.removeHeaderView(searchView.getChildAt(0));
                    if(puzzles.size() == 0){
                        noSearchResults();
                    }
                    adapter = new LazyAdapter(SearchActivity.this, puzzles);
                    searchView.setAdapter(adapter);
                    break;
            }
        }
    };

    @Override
    protected void onRestart() {
        super.onRestart();
        checkNetwork();
    }

    private void checkNetwork() {
        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        // ARE WE CONNECTED TO THE INTERNET??
        if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                && conMgr.getActiveNetworkInfo().isConnected())) {
            showDialog(NEED_INTERNET_DIALOG);
        }
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        if (id == NEED_INTERNET_DIALOG) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setMessage(R.string.search_need_internet);
            builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialogInterface, int i) {
                    dialogInterface.cancel();
                    startActivity(new Intent(getBaseContext(), MainActivity.class));
                }
            });
            builder.setPositiveButton(R.string.to_settings, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialogInterface, int i) {
                    startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
                }
            });
            return builder.create();
        }
        return null;
    }

    private void noSearchResults() {
        searchView.setAdapter(null);
        final TableRow newRow = new TableRow(SearchActivity.this);
        int textColor = getResources().getColor(R.color.app_text_color);
        float textSize = getResources().getDimension(R.dimen.fifteendp);
        addTextToRowWithValues(newRow, getResources().getString(R.string.no_puzzles), textColor, textSize);
        searchView.addHeaderView(newRow);
    }

    private void addTextToRowWithValues(final TableRow tableRow, String text, int textColor, float textSize) {
        TextView textView = new TextView(this);
        textView.setTextSize(textSize);
        textView.setTextColor(textColor);
        textView.setText(text);
        tableRow.addView(textView);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.main_options, menu);
        menu.findItem(R.id.help_menu_item).setIntent(new Intent(this, HelpActivity.class));
        menu.findItem(R.id.settings_menu_item).setIntent(new Intent(this, SettingsActivity.class));
        menu.findItem(R.id.friends_menu_item).setIntent(new Intent(this, FriendsActivity.class));
        menu.findItem(R.id.home_menu_item).setIntent(new Intent(this, MainActivity.class));
        menu.findItem(R.id.reset_menu_item).setIntent(new Intent(this, MainActivity.class));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        super.onOptionsItemSelected(item);
        startActivity(item.getIntent());
        if (item.getItemId() == R.id.reset_menu_item) {
            SharedPreferences.Editor editor = settings.edit();
            editor.remove(General.GAME_PREFERENCES_PLAYER_ID);
            editor.remove(General.GAME_PREFERENCES_NICKNAME);
            editor.remove(General.GAME_PREFERENCES_EMAIL);
            editor.commit();
        }
        return true;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(downloadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(downloadReceiver);
        }
    }
}
