package com.master.playit.helper;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

public class ContentHelper {

    private static final String PUZZLE_TABLE = "puzzle";
    private static final String FRIEND_TABLE = "friend";
    private static final String GAMEREQUEST_TABLE = "gamerequest";
    private static final String CHALLENGE_TABLE = "challenge";
    private static final String PUZZLE_LATEST_TABLE = "latest";
    private static final String AUTHOR_TABLE = "author";
    private static final String USER_TABLE = "user";
    private static final String QUESTIONSET_TABLE = "questionset";

    private final Context ourContext;
    private SQLiteDatabase ourDatabase;
    private DatabaseHelper databaseHelper;

    public ContentHelper(Context ourContext) {
        this.ourContext = ourContext;
    }

    private void open() {
        databaseHelper = new DatabaseHelper(ourContext);
        ourDatabase = databaseHelper.getWritableDatabase();
    }

    private void close() {
        databaseHelper.close();
    }

    public ArrayList<ContentValues> getLatestPuzzles() {
        open();
        Cursor latest = ourDatabase.query(PUZZLE_LATEST_TABLE, null, null, null, null, null, "playedDate DESC", "5");
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        while (latest.moveToNext()) {
            Cursor puzzle = ourDatabase.query(PUZZLE_TABLE, null, "_id = " + latest.getInt(0), null, null, null, null);
            while (puzzle.moveToNext()) {
                ContentValues cv = new ContentValues();
                // Adding each child node to ContentValues key value
                cv.put(General.XML_TAG_PUZZLE_ID, String.valueOf(puzzle.getInt(0)));
                cv.put(General.PUZZLE_NAME, puzzle.getString(1));
                cv.put(General.PUZZLE_CATEGORY, puzzle.getString(2));
                cv.put(General.XML_TAG_PUBLISHDATE, puzzle.getString(3));
                cv.put(General.XML_TAG_RATING, String.valueOf(puzzle.getInt(4)));
                cv.put(General.XML_TAG_TIMESPLAYED, String.valueOf(puzzle.getInt(5)));
                cv.put(General.XML_TAG_NUMBER_OF_QUESTIONS, String.valueOf(puzzle.getInt(6)));
                cv.put(General.PUZZLE_RATINGGIVEN, String.valueOf(latest.getInt(1)));
                cv.put(General.PUZZLE_PLAYEDDATE, String.valueOf(latest.getInt(2)));
                list.add(cv);
            }
            puzzle.close();
        }
        latest.close();
        close();
        return list;
    }

    public ContentValues getPuzzle(int puzzleId) {
        open();
        ContentValues cv = new ContentValues();
        Cursor c = ourDatabase.query(PUZZLE_TABLE, null, "_id = " + puzzleId, null, null, null, null);
        if (c.moveToFirst()) {
            cv.put(General.XML_TAG_PUZZLE_ID, String.valueOf(c.getInt(0)));
            cv.put(General.PUZZLE_NAME, c.getString(1));
            cv.put(General.PUZZLE_CATEGORY, c.getString(2));
            cv.put(General.XML_TAG_PUBLISHDATE, c.getString(3));
            cv.put(General.XML_TAG_RATING, String.valueOf(c.getInt(4)));
            cv.put(General.XML_TAG_TIMESPLAYED, String.valueOf(c.getInt(5)));
        }
        c.close();
        close();
        return cv;
    }

    public ArrayList<ContentValues> getQuestionSet(int puzzleId) {
        open();
        Cursor c = ourDatabase.query(QUESTIONSET_TABLE, null, "puzzleId = " + puzzleId, null, null, null, "setId");
        ArrayList<ContentValues> questionSet = new ArrayList<ContentValues>();
        while (c.moveToNext()) {
            ContentValues cv = new ContentValues();
            cv.put(General.XML_TAG_SET_ID, String.valueOf(c.getInt(0))); // SetId
            cv.put(General.XML_TAG_PUZZLE_ID, String.valueOf(c.getInt(1))); // PuzzleId
            cv.put(General.XML_TAG_QUESTION, c.getString(2)); // Question
            cv.put(General.XML_TAG_ANSWER, c.getString(3)); // Answer
            cv.put(General.XML_TAG_INPUTMETHOD, c.getString(4)); // InputMethod
            questionSet.add(cv);
        }
        c.close();
        close();
        return questionSet;
    }

    public ArrayList<ContentValues> getCreatedPuzzles(String nickname) {
        open();
        Cursor puzzleIds = ourDatabase.query(true, AUTHOR_TABLE, new String[]{"puzzleId"}, "nickname = \'" + nickname + "\'", null, null, null, "puzzleId", null);
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        while (puzzleIds.moveToNext()) {
            Cursor c = ourDatabase.query(PUZZLE_TABLE, null, "_id = ?", new String[]{String.valueOf(puzzleIds.getInt(0))}, null, null, null, null);
            while (c.moveToNext()) {
                ContentValues cv = new ContentValues();
                cv.put(General.XML_TAG_PUZZLE_ID, String.valueOf(c.getInt(0)));
                cv.put(General.PUZZLE_NAME, c.getString(1));
                cv.put(General.PUZZLE_CATEGORY, c.getString(2));
                cv.put(General.XML_TAG_PUBLISHDATE, c.getString(3));
                cv.put(General.XML_TAG_RATING, String.valueOf(c.getInt(4)));
                cv.put(General.XML_TAG_TIMESPLAYED, String.valueOf(c.getInt(5)));
                cv.put(General.XML_TAG_NUMBER_OF_QUESTIONS, String.valueOf(c.getInt(6)));
                list.add(cv);
            }
            c.close();
        }
        Collections.sort(list, new Comparator<ContentValues>() {
            @Override
            public int compare(ContentValues contentValues, ContentValues contentValues2) {
                String publishDate = contentValues.getAsString(General.XML_TAG_PUBLISHDATE);
                String publishDate2 = contentValues2.getAsString(General.XML_TAG_PUBLISHDATE);
                if (publishDate == null) {
                    if (publishDate2 == null || publishDate2.length() == 0) return 0;
                    return -1;
                } else if (publishDate2 == null) return 1;
                return publishDate2.compareTo(publishDate);
                // an integer < 0 if a is less than b, 0 if they are equal, and > 0 if a is greater than b.
            }
        });
        puzzleIds.close();
        close();
        return list;
    }

    public ArrayList<ContentValues> getGameRequests() {
        open();
        Cursor c = ourDatabase.query(GAMEREQUEST_TABLE, null, null, null, null, null, "requestDate DESC", null);
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        while (c.moveToNext()) {
            ContentValues cv = new ContentValues();
            // Adding each child node to ContentValues key value
            cv.put(General.ID, String.valueOf(c.getInt(0)));
            cv.put(General.XML_TAG_PUZZLE_ID, String.valueOf(c.getInt(1)));
            list.add(cv);
        }
        c.close();
        close();
        return list;
    }

    public ContentValues getGameRequest(String id) {
        open();
        Cursor c = ourDatabase.query(GAMEREQUEST_TABLE, null, "_id = " + id, null, null, null, null);
        ContentValues cv = new ContentValues();
        while (c.moveToNext()) {
            cv.put(General.ID, c.getInt(0));
            cv.put(General.XML_TAG_PUZZLE_ID, c.getInt(1));
            cv.put(General.RECEIVER_ID, c.getInt(2));
            cv.put(General.CHALLENGER_ID, c.getInt(3));
            cv.put(General.GAME_REQUESTDATE, c.getInt(4));
            cv.put(General.GAME_TYPE, c.getInt(5));
            cv.put(General.WINNER, c.getInt(6));
            cv.put(General.RECEIVER_SCORE, c.getInt(7));
            cv.put(General.CHALLENGER_SCORE, c.getInt(8));
        }
        close();
        return cv;
    }

    public ContentValues getCreatedPuzzle(int id) {
        open();
        ContentValues cv = new ContentValues();
        Cursor c = ourDatabase.query(PUZZLE_TABLE, null, "_id = ?", new String[]{String.valueOf(id)}, null, null, null);
        if (c.moveToFirst()) {
            cv.put(General.ID, c.getInt(0));
            cv.put(General.PUZZLE_NAME, c.getString(1));
            cv.put(General.PUZZLE_CATEGORY, c.getString(2));
            cv.put(General.XML_TAG_PUBLISHDATE, c.getString(3));
            cv.put(General.XML_TAG_RATING, c.getInt(4));
            cv.put(General.XML_TAG_TIMESPLAYED, c.getInt(5));
            cv.put(General.XML_TAG_NUMBER_OF_QUESTIONS, c.getInt(6));
        }
        c.close();
        close();
        return cv;
    }

    public int savePuzzleToEdit(ContentValues puzzle, ArrayList<String> authors,
                                ArrayList<ContentValues> questionSets, Integer puzzleId) {
        open();
        Integer attributesId = puzzle.getAsInteger(General.ID);
        if ((attributesId == null || attributesId == 0) && (puzzleId == null || puzzleId == 0)) {
            puzzle.remove(General.ID);
            puzzleId = (int) ourDatabase.insert(PUZZLE_TABLE, null, puzzle);
        } else {
            puzzleId = attributesId;
            ourDatabase.update(PUZZLE_TABLE, puzzle, "_id = " + puzzleId, null);
        }
        saveAuthorsAndQuestionSet(authors, questionSets, puzzleId);
        close();
        return puzzleId;
    }

    private void saveAuthorsAndQuestionSet(ArrayList<String> authors, ArrayList<ContentValues> questionSets, Integer puzzleId) {
        if (questionSets != null) {
            for (ContentValues cv : questionSets) {
                cv.put(General.XML_TAG_PUZZLE_ID, puzzleId);
                Integer setId = cv.getAsInteger(General.XML_TAG_SET_ID);
                Cursor c = ourDatabase.query(QUESTIONSET_TABLE, new String[]{"setId"}, "setId = " + setId, null, null, null, null);
                if (c.moveToNext()) {
                    ourDatabase.update(QUESTIONSET_TABLE, cv, "setId = " + setId, null);
                } else {
                    ourDatabase.insert(QUESTIONSET_TABLE, null, cv);
                }
            }
        }
        ContentValues cv = new ContentValues();
        Cursor authorExists = ourDatabase.query(AUTHOR_TABLE, null, "puzzleId = ? AND nickname = ?",
                new String[]{String.valueOf(puzzleId), authors.get(0)}, null, null, null);
        if (!authorExists.moveToNext()) {
            cv.put(General.XML_TAG_NICKNAME, authors.get(0));
            cv.put(General.MAIN_AUTHOR, 1);
            cv.put(General.XML_TAG_PUZZLE_ID, puzzleId);
            ourDatabase.insert(AUTHOR_TABLE, null, cv);
        }
        authors.remove(0);

        if (authors.size() > 0) {
            for (String author : authors) {
                authorExists = ourDatabase.query(AUTHOR_TABLE, null, "puzzleId = ? AND nickname = ?",
                        new String[]{String.valueOf(puzzleId), authors.get(0)}, null, null, null);
                if (!authorExists.moveToNext()) {
                    cv.clear();
                    cv.put(General.XML_TAG_NICKNAME, author);
                    cv.put(General.MAIN_AUTHOR, 0);
                    cv.put(General.XML_TAG_PUZZLE_ID, puzzleId);
                    ourDatabase.insert(AUTHOR_TABLE, null, cv);
                }
            }
        }
    }

    public void saveCreatedPuzzles(ArrayList<ContentValues> list, String nickname) {
        open();
        for (ContentValues cv : list) {
            ContentValues author = new ContentValues();
            Integer puzzleId = cv.getAsInteger(General.ID);
            author.put(General.XML_TAG_PUZZLE_ID, puzzleId);
            author.put(General.MAIN_AUTHOR, 1);
            author.put(General.XML_TAG_NICKNAME, nickname);
            Cursor puzzle = ourDatabase.query(PUZZLE_TABLE, new String[]{"_id"}, "_id = " + puzzleId, null, null, null, null);
            if (!puzzle.moveToFirst()) {
                ourDatabase.insert(PUZZLE_TABLE, null, cv);
                ourDatabase.insert(AUTHOR_TABLE, null, author);
            } else {
                ourDatabase.update(PUZZLE_TABLE, cv, "_id = " + puzzleId, null);
                ourDatabase.update(AUTHOR_TABLE, author, "puzzleId = " + puzzleId, null);
            }
            puzzle.close();
        }
        close();
    }

    public void publishPuzzleLocally(int puzzleId, int newId) {
        if (puzzleId != newId) {
            updatePuzzleId(puzzleId, newId);
            puzzleId = newId;
        }
        // Set puzzle published locally only
        open();
        ContentValues cv = new ContentValues();
        Date date = new Date(new java.util.Date().getTime());
        cv.put(General.XML_TAG_PUBLISHDATE, date.toString());
        cv.put(General.XML_TAG_RATING, 0);
        cv.put(General.XML_TAG_TIMESPLAYED, 0);
        ourDatabase.update(PUZZLE_TABLE, cv, "_id = " + puzzleId, null);
        close();
    }

    public void saveQuestionSet(int puzzleId, ArrayList<ContentValues> questionSet) {
        // Delete questionSet then update
        open();
        ourDatabase.delete(QUESTIONSET_TABLE, "puzzleId = " + puzzleId, null);
        for (int i = 0; i < questionSet.size(); i++) {
            questionSet.get(i).put(General.XML_TAG_SET_ID, i + 1);
            questionSet.get(i).put(General.XML_TAG_PUZZLE_ID, puzzleId);
            ourDatabase.insert(QUESTIONSET_TABLE, null, questionSet.get(i));
        }
        close();
    }

    public ArrayList<String> getCoAuthors(int puzzleId) {
        open();
        ArrayList<String> coAuthors = new ArrayList<String>();
        Cursor c = ourDatabase.query(AUTHOR_TABLE, new String[]{"nickname"}, "puzzleId = " + puzzleId + " AND mainAuthor = " + 0, null, null, null, null);
        while (c.moveToNext()) {
            coAuthors.add(c.getString(0));
        }
        c.close();
        close();
        return coAuthors;
    }

    public void updatePuzzleId(int oldId, int newId) {
        open();
        Cursor c = ourDatabase.query(PUZZLE_TABLE, new String[]{"publishdate"}, "_id = " + newId, null, null, null, null);
        int newId2 = newId + 1;
        if (c.moveToFirst()) {                                    // NewId exists
            if (c.getString(0) != null) {                        // NewId's puzzle has not been published (of course)
                updatePuzzleId(newId, newId2);                  // Change to newId2 recursively until an available id is found
            } else {
                updatePuzzleId(oldId, newId2);
            }
        }
        ContentValues cv = new ContentValues();
        cv.put(General.ID, newId);                                              // Switch oldId to newId
        ourDatabase.update(PUZZLE_TABLE, cv, "_id = " + oldId, null);
        cv.clear();
        cv.put(General.XML_TAG_PUZZLE_ID, newId);
        int rows = ourDatabase.update(AUTHOR_TABLE, cv, "puzzleId = " + oldId, null);
        ourDatabase.update(QUESTIONSET_TABLE, cv, "puzzleId = " + oldId, null);
        close();
    }

    public void addQuestionSet(ArrayList<ContentValues> list) {
        open();
        for (ContentValues cv : list) {
            ourDatabase.insert(QUESTIONSET_TABLE, null, cv);
        }
        close();
    }

    public boolean isMainAuthor(int puzzleId) {
        open();
        Cursor c = ourDatabase.query(AUTHOR_TABLE, null, "puzzleId = " + puzzleId + " AND mainAuthor = " + 1, null, null, null, null);
        if (c.moveToFirst()) {
            return true;
        }
        c.close();
        close();
        return false;
    }

    public void saveGameRequests(ArrayList<ContentValues> requestList) {
        open();
        for (ContentValues cv : requestList) {
            int lines = ourDatabase.update(GAMEREQUEST_TABLE, cv, "_id = " + cv.getAsString(General.ID), null);
            if (lines == 0) {
                ourDatabase.insert(GAMEREQUEST_TABLE, null, cv);
            }
        }
        close();
    }

    public void removeAuthor(int puzzleId, String nickname) {
        open();
        Cursor c = ourDatabase.query(FRIEND_TABLE, new String[]{"_id"}, "nickname = ?", new String[]{nickname}, null, null, null);
        c.moveToFirst();
        ourDatabase.delete(AUTHOR_TABLE, "puzzleId = " + puzzleId + " AND userId = " + c.getInt(0), null);
        c.close();
        close();
    }

    public ArrayList<String> getAuthors(int puzzleId) {
        open();
        ArrayList<String> nicknames = new ArrayList<String>();
        Cursor c = ourDatabase.query(AUTHOR_TABLE, new String[]{"nickname"}, "puzzleId = " + puzzleId, null, null, null, "nickname");
        while (c.moveToNext()) {
            nicknames.add(c.getString(0));
        }
        c.close();
        close();
        return nicknames;
    }

    public void savePuzzleToPlay(ContentValues puzzleAttributes, ArrayList<String> authors,
                                 ArrayList<ContentValues> questionSets, int puzzleId) {
        open();
        Cursor puzzle = ourDatabase.query(PUZZLE_TABLE, new String[]{"_id"}, "_id = " + puzzleId, null, null, null, null);
        if (!puzzle.moveToFirst()) {
            ourDatabase.insert(PUZZLE_TABLE, null, puzzleAttributes);
        }
        saveAuthorsAndQuestionSet(authors, questionSets, puzzleId);
        close();
    }

    public boolean puzzleExistsAndIsSaved(int puzzleId) {
        open();
        boolean exists = false;
        Cursor puzzle = ourDatabase.query(PUZZLE_TABLE, new String[]{"_id"}, "_id = " + puzzleId, null, null, null, null);
        Cursor author = ourDatabase.query(AUTHOR_TABLE, new String[]{"nickname"}, "puzzleId = " + puzzleId, null, null, null, null);
        Cursor questionSet = ourDatabase.query(QUESTIONSET_TABLE, new String[]{"setId"}, "puzzleId = " + puzzleId, null, null, null, "setId");
        if (puzzle.moveToFirst() && author.moveToFirst() && questionSet.moveToFirst()) exists = true;
        puzzle.close();
        author.close();
        questionSet.close();
        close();
        return exists;
    }

    public boolean isPublished(int puzzleId) {
        open();
        Cursor c = ourDatabase.query(PUZZLE_TABLE, new String[]{"publishdate"}, "_id = " + puzzleId, null, null, null, null);
        boolean published = false;
        if (c.moveToNext()) {
            String publishDate = c.getString(0);
            if (publishDate != null && publishDate.length() > 4) published = true;
        }
        c.close();
        close();
        return published;
    }

    public void saveLatestPlayedPuzzle(int puzzleId, Integer ratingGiven, String response) {
        open();
        ContentValues cv = new ContentValues();
        cv.put(General.XML_TAG_PUZZLE_ID, puzzleId);
        cv.put(General.PUZZLE_PLAYEDDATE, new java.util.Date().toString());
        Cursor puzzle = ourDatabase.query(PUZZLE_LATEST_TABLE, new String[]{"puzzleId"}, "puzzleId = " + puzzleId, null, null, null, null);
        if (puzzle.moveToFirst()) {
            ourDatabase.update(PUZZLE_LATEST_TABLE, cv, "puzzleId = " + puzzleId, null);
        } else {
            cv.put(General.PUZZLE_RATINGGIVEN, ratingGiven != null ? ratingGiven : -1);
            ourDatabase.insert(PUZZLE_LATEST_TABLE, null, cv);
        }
        cv.clear();
        if (response != null) {
            String[] responses = response.split(",");
            cv.put(General.XML_TAG_RATING, responses[0]);
            cv.put(General.XML_TAG_TIMESPLAYED, responses[1]);
            ourDatabase.update(PUZZLE_TABLE, cv, "_id = " + puzzleId, null);
        }
        puzzle.close();
        close();
    }

    public boolean isPuzzleRated(int puzzleId) {
        open();
        boolean rated = false;
        Cursor c = ourDatabase.query(PUZZLE_LATEST_TABLE, new String[]{"ratingGiven"}, "puzzleId = " + puzzleId, null, null, null, null);
        if (c.moveToFirst()) {
            int ratingGiven = c.getInt(0);
            if (ratingGiven > -1) rated = true;
        }
        close();
        return rated;
    }

    public void addTimesPlayed(int puzzleId) {
        open();
        Cursor c = ourDatabase.query(PUZZLE_TABLE, new String[]{"timesplayed"}, "_id = " + puzzleId, null, null, null, null);
        if (c.moveToFirst()) {
            ContentValues cv = new ContentValues();
            cv.put(General.XML_TAG_TIMESPLAYED, c.getInt(0) + 1);
            ourDatabase.update(PUZZLE_TABLE, cv, "_id = " + puzzleId, null);
        }
        c.close();
        close();
    }

    public void saveFriends(ArrayList<ContentValues> friends) {
        open();
        ourDatabase.delete(FRIEND_TABLE, null, null);
        for (ContentValues friend : friends) {
            ourDatabase.insert(FRIEND_TABLE, null, friend);
        }
        close();
    }

    public boolean hasCollaborators(int id) {
        open();
        boolean collaborators = false;
        Cursor c = ourDatabase.query(AUTHOR_TABLE, new String[]{"nickname"}, "puzzleId = ? AND mainAuthor = ?", new String[]{String.valueOf(id), String.valueOf(0)}, null, null, null, null);
        if (c.moveToFirst()) {
            collaborators = true;
        }
        close();
        return collaborators;
    }

    private static class DatabaseHelper extends SQLiteOpenHelper {
        private static final String TAG = "SQLiteOpenHelper";
        private static final int DATABASE_VERSION = 2;
        private static final String DATABASE_NAME = "puzzles.db";
        private static final String PUZZLE_TABLE_CREATE =
                "CREATE TABLE puzzle ( _id INTEGER PRIMARY KEY, name varchar(45), category varchar(25), " +
                        "publishdate DATE, rating INT, timesplayed INT, numberOfQuestions INT );";
        private static final String FRIEND_TABLE_CREATE =
                "CREATE TABLE friend ( _id INTEGER PRIMARY KEY, nickname VARCHAR(20) );";
        private static final String GAMEREQUEST_TABLE_CREATE =
                "CREATE TABLE gamerequest ( _id INTEGER PRIMARY KEY, puzzleId INT, receiverId INT, " +
                        "challengerId INT, requestDate DATE, type varchar(20), winner INT, receiverScore INT, challengeScore INT );";
        private static final String CHALLENGE_TABLE_CREATE =
                "CREATE TABLE challenge (_challengeId INTEGER PRIMARY KEY, puzzleId INT, score INT, receiverId INT, challengeDate DATE );";
        private static final String AUTHOR_TABLE_CREATE =
                "CREATE TABLE author( puzzleId INT, nickname VARCHAR(30), mainAuthor BIT );";
        private static final String PUZZLE_LATEST_TABLE_CREATE =
                "CREATE TABLE latest( puzzleId INT, ratingGiven INT, playedDate DATE, score INT, totalTime varchar(10) );";   // hh:mm:ss
        private static final String QUESTIONSET_TABLE_CREATE =
                "CREATE TABLE questionset( setId INT, puzzleId INT, question varchar(55), answer varchar(45), inputmethod varchar(20) );";

        public DatabaseHelper(Context context) {
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL(PUZZLE_TABLE_CREATE);
            db.execSQL(FRIEND_TABLE_CREATE);
            db.execSQL(GAMEREQUEST_TABLE_CREATE);
            db.execSQL(CHALLENGE_TABLE_CREATE);
            db.execSQL(AUTHOR_TABLE_CREATE);
            db.execSQL(PUZZLE_LATEST_TABLE_CREATE);
            db.execSQL(QUESTIONSET_TABLE_CREATE);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
                    + newVersion + ", which will destroy all old data");
            db.execSQL("DROP TABLE IF EXISTS" + PUZZLE_LATEST_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + PUZZLE_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + AUTHOR_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + QUESTIONSET_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + GAMEREQUEST_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + CHALLENGE_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + FRIEND_TABLE);
            db.execSQL("DROP TABLE IF EXISTS" + USER_TABLE);
            onCreate(db);
        }
    }

    public void addFriend(ContentValues cv) {
        open();
        ourDatabase.insert(FRIEND_TABLE, null, cv);
        close();
    }

    public void deleteFriends() {
        open();
        ourDatabase.delete(FRIEND_TABLE, null, null);
        close();
    }

    public ArrayList<String> getFriends() {
        open();
        ArrayList<String> friends = new ArrayList<String>();
        Cursor c = ourDatabase.query(FRIEND_TABLE, new String[]{"nickname"}, null, null, null, null, "nickname");
        while (c.moveToNext()) {
            friends.add(c.getString(0));
        }
        close();
        return friends;
    }

    public void deleteGameRequest(String id) {
        open();
        ourDatabase.delete(GAMEREQUEST_TABLE, "_id = " + id, null);
        close();
    }

    public void deletePuzzle(int id) {
        open();
        ourDatabase.delete(PUZZLE_TABLE, "_id = " + id, null);
        ourDatabase.delete(QUESTIONSET_TABLE, "puzzleId = " + id, null);
        ourDatabase.delete(AUTHOR_TABLE, "puzzleId = " + id, null);
        close();
    }

    public void deleteGameRequests() {
        open();
        ourDatabase.delete(GAMEREQUEST_TABLE, null, null);
        close();
    }

    public void deleteLatestPuzzle(String id) {
        open();
        ourDatabase.delete(PUZZLE_LATEST_TABLE, "puzzleId = " + id, null);
        ourDatabase.delete(QUESTIONSET_TABLE, "_id = " + id, null);
        close();
    }
}
