package com.master.playit.helper;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;
import com.master.playit.R;

import java.util.ArrayList;

public class LazyAdapter extends BaseAdapter {

    private ArrayList<ContentValues> data;
    private LayoutInflater inflater;

    public LazyAdapter(Activity activity, ArrayList<ContentValues> data) {
        this.data = data;
        this.inflater = (LayoutInflater) activity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    @Override
    public int getCount() {
        return data.size();
    }

    @Override
    public Object getItem(int position) {
        return position;
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ContentValues attributes = data.get(position);
        if (convertView == null)
            convertView = new View(parent.getContext());

        if (attributes.size() == 8) {
            convertView = inflater.inflate(R.layout.list_row_latest, null);
            if (data.size() == 0) {
                TextView name = (TextView) convertView.findViewById(R.id.name);
                name.setText(R.string.no_played_puzzles);
            }
        } else {
            if (!attributes.containsKey(General.GAME_REQUESTDATE)) {
                convertView = inflater.inflate(R.layout.list_row_created, null);
                if (data.size() == 0) {
                    TextView name = (TextView) convertView.findViewById(R.id.name);
                    name.setText(R.string.no_created_puzzles);
                }
            } else {
                convertView = inflater.inflate(R.layout.list_row_request, null);
                if (data.size() == 0) {
                    TextView name = (TextView) convertView.findViewById(R.id.name);
                    name.setText(R.string.no_game_requests);
                }
                TextView name = (TextView) convertView.findViewById(R.id.name);
                TextView category = (TextView) convertView.findViewById(R.id.category);
                TextView requestDate = (TextView) convertView.findViewById(R.id.requestDate);
                TextView type = (TextView) convertView.findViewById(R.id.type);
                TextView scoreToBeat = (TextView) convertView.findViewById(R.id.scoreToBeat);

                convertView.setTag(R.id.ID_TAG, attributes.get(General.ID));
                convertView.setTag(R.id.NAME_TAG, attributes.get(General.PUZZLE_NAME));

                name.setText(attributes.getAsString(General.PUZZLE_NAME));
                category.setText(attributes.getAsString(General.PUZZLE_CATEGORY));
                requestDate.setText(attributes.getAsString(General.GAME_REQUESTDATE));
                type.setText(attributes.getAsString(General.GAME_TYPE));
                scoreToBeat.setText(attributes.getAsString(General.GAME_CHALLENGESCORE));
                return convertView;
            }
        }
        if(attributes.containsKey(General.ID))
            convertView.setTag(R.id.ID_TAG, attributes.get(General.ID));
        if(attributes.containsKey(General.XML_TAG_PUZZLE_ID))
            convertView.setTag(R.id.ID_TAG, attributes.get(General.XML_TAG_PUZZLE_ID));

        convertView.setTag(R.id.NAME_TAG, attributes.get(General.PUZZLE_NAME));

        TextView name = (TextView) convertView.findViewById(R.id.name);
        TextView category = (TextView) convertView.findViewById(R.id.category);
        TextView rating = (TextView) convertView.findViewById(R.id.rating);
        TextView publishDate = (TextView) convertView.findViewById(R.id.publishDate);
        TextView timesPlayed = (TextView) convertView.findViewById(R.id.timesPlayed);
        TextView questions = (TextView) convertView.findViewById(R.id.questions);

        if (attributes.size() == 8) {
            // LatestPuzzle
            TextView ratingGiven = (TextView) convertView.findViewById(R.id.myRating);
            ratingGiven.setText("My rating: " + attributes.getAsString(General.PUZZLE_RATINGGIVEN));
        }

        name.setText(attributes.getAsString(General.PUZZLE_NAME));
        category.setText(attributes.getAsString(General.PUZZLE_CATEGORY));
        questions.setText(attributes.getAsString(General.XML_TAG_NUMBER_OF_QUESTIONS) + " questions");

        String dateString = attributes.getAsString(General.XML_TAG_PUBLISHDATE);
        if (dateString == null || dateString.length() == 0) {
            rating.setText(R.string.line);
            publishDate.setText(R.string.not_published);
            timesPlayed.setText(R.string.line);
        } else {
            rating.setText("Rating: " +  attributes.getAsString(General.XML_TAG_RATING));
            publishDate.setText(dateString);
            timesPlayed.setText("#Played: " +  attributes.getAsString(General.XML_TAG_TIMESPLAYED));
        }
        return convertView;
    }
}
