package com.master.playit.helper;

import android.content.ContentValues;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.resource.ClientResource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.IOException;
import java.util.ArrayList;

public class XmlParser {

    // Constructor
    public XmlParser() {

    }

    /**
     * Getting XML from URL making HTTP request
     *
     * @param url string
     */
    public Document getDocFromUrl(String url) {
        Document doc = null;
        try {
            // DefaultHttpClient
            ClientResource resource = new ClientResource(url);
            DomRepresentation rep = new DomRepresentation(resource.get());

            doc = rep.getDocument();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return doc;
    }

    /**
     * Getting node value
     *
     * @param elem element
     */
    public final String getElementValue(Node elem) {
        Node child;
        if (elem != null) {
            if (elem.hasChildNodes()) {
                for (child = elem.getFirstChild(); child != null; child = child.getNextSibling()) {
                    if (child.getNodeType() == Node.TEXT_NODE) {
                        return child.getNodeValue();
                    }
                }
            }
        }
        return "";
    }

    /**
     * Getting node value
     *
     * @param item node
     * @param str  string
     */
    public String getValue(Element item, String str) {
        NodeList n = item.getElementsByTagName(str);
        return this.getElementValue(n.item(0));
    }

    public ArrayList<ContentValues> getPuzzles(Document doc) {
        NodeList createdNodes = doc.getElementsByTagName(General.XML_TAG_PUZZLE);
        // Looping through all nodes
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        for (int i = 0; i < createdNodes.getLength(); i++) {
            // Creating new ContentValues
            ContentValues cv = new ContentValues();
            Element e = (Element) createdNodes.item(i);
            // Adding each child node to ContentValues key value
            cv.put(General.ID, getValue(e, General.XML_TAG_ID));
            cv.put(General.PUZZLE_NAME, getValue(e, General.PUZZLE_NAME));
            cv.put(General.PUZZLE_CATEGORY, getValue(e, General.PUZZLE_CATEGORY));
            cv.put(General.XML_TAG_RATING, getValue(e, General.XML_TAG_RATING));
            cv.put(General.XML_TAG_PUBLISHDATE, getValue(e, General.XML_TAG_PUBLISHDATE));
            cv.put(General.XML_TAG_TIMESPLAYED, getValue(e, General.XML_TAG_TIMESPLAYED));
            cv.put(General.XML_TAG_NUMBER_OF_QUESTIONS, getValue(e, General.XML_TAG_NUMBER_OF_QUESTIONS));
            // Adding ContentValues to ArrayList
            list.add(cv);
        }
        return list;
    }

    public ArrayList<ContentValues> getGameRequests(NodeList requestNodes) {
        ArrayList<ContentValues> requestList = new ArrayList<ContentValues>();
        // Looping through all nodes;
        for (int i = 0; i < requestNodes.getLength(); i++) {
            // Creating new ContentValues
            ContentValues cv = new ContentValues();
            Element e = (Element) requestNodes.item(i);
            // Adding each child node to key ContentValues value
            cv.put(General.ID, getValue(e, General.XML_TAG_REQUEST_ID));
            cv.put(General.XML_TAG_PUZZLE_ID, getValue(e, General.XML_TAG_PUZZLE_ID));
            cv.put(General.PUZZLE_NAME, getValue(e, General.PUZZLE_NAME));
            cv.put(General.PUZZLE_CATEGORY, getValue(e, General.PUZZLE_CATEGORY));
            cv.put(General.GAME_REQUESTDATE, getValue(e, General.GAME_REQUESTDATE));
            cv.put(General.GAME_TYPE, getValue(e, General.GAME_TYPE));
            cv.put(General.GAME_CHALLENGESCORE, getValue(e, General.GAME_CHALLENGESCORE));
            requestList.add(cv);
        }
        return requestList;
    }

    public ContentValues getPuzzle(Element puzzle) {
        // Creating new ContentValues
        ContentValues cv = new ContentValues();
        // Adding each child node to ContentValues key value
        cv.put(General.ID, getValue(puzzle, General.XML_TAG_ID));
        cv.put(General.PUZZLE_NAME, getValue(puzzle, General.PUZZLE_NAME));
        cv.put(General.PUZZLE_CATEGORY, getValue(puzzle, General.PUZZLE_CATEGORY));
        cv.put(General.XML_TAG_RATING, getValue(puzzle, General.XML_TAG_RATING));
        cv.put(General.XML_TAG_PUBLISHDATE, getValue(puzzle, General.XML_TAG_PUBLISHDATE));
        cv.put(General.XML_TAG_TIMESPLAYED, getValue(puzzle, General.XML_TAG_TIMESPLAYED));
        cv.put(General.XML_TAG_NUMBER_OF_QUESTIONS, getValue(puzzle, General.XML_TAG_NUMBER_OF_QUESTIONS));
        return cv;
    }

    public ArrayList<String> getAuthors(Element puzzleElement) {
        ArrayList<String> authors = new ArrayList<String>();
        String author = getValue(puzzleElement, General.XML_TAG_AUTHOR);
        authors.add(author);
        NodeList coAuthors = puzzleElement.getElementsByTagName(General.XML_TAG_COAUTHORS);
        for (int i = 0; i < coAuthors.getLength(); i++) {
            authors.add(getValue((Element) coAuthors.item(i), General.XML_TAG_COAUTHOR));
        }
        return authors;
    }

    public ArrayList<ContentValues> getQuestionSet(Element doc) {
        NodeList questionSetNodes = doc.getElementsByTagName(General.QUESTIONSET);
        // Looping through all nodes
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        for (int i = 0; i < questionSetNodes.getLength(); i++) {
            // Creating new ContentValues
            ContentValues cv = new ContentValues();
            Element e = (Element) questionSetNodes.item(i);
            // Adding each child node to ContentValues key value
            cv.put(General.XML_TAG_SET_ID, getValue(e, General.XML_TAG_SET_ID));
            cv.put(General.XML_TAG_QUESTION, getValue(e, General.XML_TAG_QUESTION));
            cv.put(General.XML_TAG_ANSWER, getValue(e, General.XML_TAG_ANSWER));
            cv.put(General.XML_TAG_INPUTMETHOD, getValue(e, General.XML_TAG_INPUTMETHOD));
            // Adding ContentValues to ArrayList
            list.add(cv);
        }
        return list;
    }

    public ArrayList<ContentValues> getScores(Document doc) {
        NodeList scores = doc.getElementsByTagName(General.XML_TAG_SCORE);
        // Looping through all nodes
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        for (int i = 0; i < scores.getLength(); i++) {
            // Creating new ContentValues
            ContentValues score = new ContentValues();
            Element e = (Element) scores.item(i);
            // Adding each child node to ContentValues key value
            score.put(General.XML_TAG_PUZZLE_ID, getValue(e, General.XML_TAG_PUZZLE_ID));
            score.put(General.XML_TAG_NICKNAME, getValue(e, General.XML_TAG_NICKNAME));
            score.put(General.SCORE, getValue(e, General.XML_TAG_VALUE));
            score.put(General.TIME, getValue(e, General.XML_TAG_TIME));
            // Adding ContentValues to ArrayList
            list.add(score);
        }
        return list;
    }

    public ArrayList<ContentValues> getFriends(Document doc) {
        NodeList friends = doc.getElementsByTagName("friend");
        ArrayList<ContentValues> list = new ArrayList<ContentValues>();
        for(int i = 0; i < friends.getLength(); i++){
            ContentValues cv = new ContentValues();
            Element friend = (Element) friends.item(i);
            cv.put(General.XML_TAG_NICKNAME, getValue(friend, General.GAME_PREFERENCES_NICKNAME));
            cv.put(General.ID, getValue(friend, General.XML_TAG_ID));
            list.add(cv);
        }
        return list;
    }
}