package com.master.playit;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.*;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.*;
import com.master.playit.helper.ContentHelper;
import com.master.playit.helper.General;
import com.master.playit.helper.LazyAdapter;
import com.master.playit.helper.UpdaterService;
import org.restlet.representation.EmptyRepresentation;
import org.restlet.resource.ClientResource;
import roboguice.activity.RoboActivity;
import roboguice.inject.InjectView;

import java.util.ArrayList;

public class MainActivity extends RoboActivity {

    @InjectView(R.id.TabHost_Main)
    TabHost main;
    @InjectView(R.id.ListView_Created)
    ListView createdView;
    @InjectView(R.id.ListView_Latest)
    ListView latestView;
    @InjectView(R.id.ListView_Request)
    ListView requestsView;

    private static final int CREATED_PUZZLE_DIALOG = 1;
    private static final int GAME_REQUEST_DIALOG = 2;
    private static final int LATEST_PUZZLE_DIALOG = 3;
    private static final int YOU_SURE_DELETE_DIALOG = 4;
    private static final int NEED_INTERNET_DIALOG = 5;
    public static final int FETCH_CREATED_PUZZLES = 6;
    public static final int FETCH_GAME_REQUESTS = 7;
    private static final int PROGRESS_DIALOG = 8;
    private static final int PUBLISH_PUZZLE = 9;
    private static final int CREATED_PUBLISH_PUZZLE_DIALOG = 10;
    public static final int DELETE_PUZZLE = 12;

    private SharedPreferences settings;
    private SharedPreferences defaultPreferences;
    private LazyAdapter adapter;
    private ContentHelper helper;
    private boolean receiverRegistered = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        settings = getSharedPreferences(General.GAME_PREFERENCES, Context.MODE_PRIVATE);
        defaultPreferences = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        helper = new ContentHelper(this);
        boolean useLocalDataAtStartUp = defaultPreferences.getBoolean(General.START_UP, false);
        boolean useWifiIfAvailable = defaultPreferences.getBoolean(General.WIFI, true);
        boolean useThreeGIfAvailable = defaultPreferences.getBoolean(General.THREEG, false);

        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        boolean isWifiEnabled = conMgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI).isAvailable();
        boolean is3GEnabled = !(conMgr.getNetworkInfo(ConnectivityManager.TYPE_MOBILE).getState() == NetworkInfo.State.DISCONNECTED);

        initTabs();

        startUpdates(false);

//        showDialog(PROGRESS_DIALOG);
        // ARE WE CONNECTED TO THE INTERNET?
//        boolean notConnectedToTheInternet = !(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable() && conMgr.getActiveNetworkInfo().isConnected());
//        if (useLocalDataAtStartUp) {
//            startUpdates(false);
//        } else if (notConnectedToTheInternet) {
//            showDialog(NEED_INTERNET_DIALOG);
//        } else if (!useWifiIfAvailable && isWifiEnabled) {
//            startUpdates(false);
//        } else if (!useThreeGIfAvailable && is3GEnabled) {
//            startUpdates(false);
//        } else {
//            startUpdates(true);
//        }
        addButtonsToLists();
    }

    private void addButtonsToLists() {
        TableRow latestRow = new TableRow(getBaseContext());
        TextView latestTextView = new TextView(getBaseContext());
        latestTextView.setPadding(0, 10, 0, 10);
        latestTextView.setGravity(1);
        latestTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 18);

        latestTextView.setText(R.string.play_new_puzzle);
        latestTextView.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                startActivity(new Intent(view.getContext(), SearchActivity.class));
            }
        });
        latestRow.addView(latestTextView);
        latestView.addFooterView(latestRow);

        TableRow createdRow = new TableRow(getBaseContext());
        TextView createdTextView = new TextView(getBaseContext());
        createdTextView.setPadding(0, 10, 0, 10);
        createdTextView.setGravity(1);
        createdTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 18);

        createdTextView.setText(R.string.create_new_puzzle);
        createdTextView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                startActivity(new Intent(getBaseContext(), CreateActivity.class));
                SharedPreferences.Editor editor = settings.edit();
                editor.remove(General.GAME_PREFERENCES_CURRENT_PUZZLE);
                editor.commit();
            }
        });
        createdRow.addView(createdTextView);
        createdView.addFooterView(createdRow);
    }

    private void setOnItemClickListeners() {
        // Click event for single list row
        createdView.setOnItemClickListener(new OnItemClick(CreateActivity.class));
        createdView.setOnItemLongClickListener(new OnItemLongClick(CREATED_PUZZLE_DIALOG));
        latestView.setOnItemClickListener(new OnItemClick(MainPlayActivity.class));
        latestView.setOnItemLongClickListener(new OnItemLongClick(LATEST_PUZZLE_DIALOG));
        requestsView.setOnItemClickListener(new OnItemClick(MainPlayActivity.class));
        requestsView.setOnItemLongClickListener(new OnItemLongClick(GAME_REQUEST_DIALOG));
    }

    private class OnItemClick implements AdapterView.OnItemClickListener {
        private Class<?> cls;

        private OnItemClick(Class<?> cls) {
            this.cls = cls;
        }

        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            String puzzleId = view.getTag(R.id.ID_TAG).toString();
            if (helper.isPublished(Integer.parseInt(puzzleId))) {
                cls = MainPlayActivity.class;
                Intent intent = new Intent(getBaseContext(), cls);
                intent.putExtra(General.ID, Integer.parseInt(puzzleId));
                startActivity(intent);
            } else {
                Intent intent = new Intent(getBaseContext(), cls);
                intent.putExtra(General.ID, Integer.parseInt(puzzleId));
                startActivity(intent);
            }
        }
    }

    private class OnItemLongClick implements AdapterView.OnItemLongClickListener {
        private int dialog;

        private OnItemLongClick(int dialog) {
            this.dialog = dialog;
        }

        @Override
        public boolean onItemLongClick(AdapterView<?> adapterView, View view, int i, long l) {
            Bundle bundle = new Bundle();
            if (view.getTag(R.id.ID_TAG) != null) {
                String puzzleId = view.getTag(R.id.ID_TAG).toString();
                bundle.putInt(General.ID, Integer.parseInt(puzzleId));
                bundle.putString(General.PUZZLE_NAME, view.getTag(R.id.NAME_TAG).toString());
                boolean published = helper.isPublished(Integer.parseInt(puzzleId));
                if (published) {
                    dialog = CREATED_PUBLISH_PUZZLE_DIALOG;
                }
                showDialog(dialog, bundle);
                return true;
            }
            return false;
        }
    }

    private void startUpdates(boolean internet) {
        if (internet) {
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.FETCH_CREATED_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            intentFilter.addAction(UpdaterService.FETCH_GAME_REQUEST_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            receiverRegistered = true;

            Intent intent = new Intent(getBaseContext(), UpdaterService.class);
            intent.putExtra(General.GAME_PREFERENCES_PLAYER_ID, settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0));
            intent.putExtra(General.UPDATER_ACTION, FETCH_CREATED_PUZZLES);
            startService(intent);
            intent.putExtra(General.UPDATER_ACTION, FETCH_GAME_REQUESTS);
            startService(intent);
        } else {
            fillListViews();
        }
    }

    @Override
    protected void onPrepareDialog(int id, Dialog dialog, final Bundle bundle) {
        switch (id) {
            case CREATED_PUZZLE_DIALOG:
                dialog.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                break;
            case CREATED_PUBLISH_PUZZLE_DIALOG:
                dialog.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                break;
            case DELETE_PUZZLE:
                fillListViews();
                break;
        }
        super.onPrepareDialog(id, dialog, bundle);
    }

    @Override
    protected Dialog onCreateDialog(int id, final Bundle bundle) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        switch (id) {
            case CREATED_PUZZLE_DIALOG:
                builder.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                builder.setItems(R.array.created_puzzle, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int which) {
                        if (which == 0) { // = edit
                            Intent intent = new Intent(getBaseContext(), CreateActivity.class);
                            intent.putExtras(bundle);
                            startActivity(intent);
                        } else if (which == 1) { // = publish
                            showDialog(PUBLISH_PUZZLE, bundle);
                        } else if (which == 2) { // = delete
                            if (helper.isMainAuthor(bundle.getInt(General.ID)))
                                showDialog(YOU_SURE_DELETE_DIALOG, bundle);
                            else
                                Toast.makeText(getBaseContext(), R.string.main_author_delete, Toast.LENGTH_SHORT).show();
                        }
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                return builder.create();
            case CREATED_PUBLISH_PUZZLE_DIALOG:
                builder.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                builder.setItems(R.array.published_created_puzzle, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int which) {
                        if (which == 0) { // = play
                            Intent intent = new Intent(getBaseContext(), MainPlayActivity.class);
                            intent.putExtras(bundle);
                            SharedPreferences.Editor editor = settings.edit();
                            editor.putInt(General.GAME_PREFERENCES_CURRENT_QUESTION, 0);
                            editor.commit();
                            startActivity(intent);
                        } else if (which == 1) { // = scores
                            Intent intent = new Intent(getBaseContext(), ScoreActivity.class);
                            intent.putExtras(bundle);
                            startActivity(intent);
                        } else if (which == 2) { // = delete
                            if (helper.isMainAuthor(bundle.getInt(General.ID)))
                                showDialog(YOU_SURE_DELETE_DIALOG, bundle);
                            else
                                Toast.makeText(getBaseContext(), R.string.main_author_delete, Toast.LENGTH_SHORT).show();
                        }
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                return builder.create();
            case LATEST_PUZZLE_DIALOG:
                builder.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                builder.setItems(R.array.latest_puzzle, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int which) {
                        if (which == 0) { // = play
                            Intent intent = new Intent(getBaseContext(), MainPlayActivity.class);
                            intent.putExtras(bundle);
                            startActivity(intent);
                        } else if (which == 1) { // = scores
                            Intent intent = new Intent(getBaseContext(), ScoreActivity.class);
                            intent.putExtras(bundle);
                            startActivity(intent);
                        } else if (which == 2) { // = remove
                            deleteLatestPuzzle(bundle.getString(General.ID));
                            startActivity(new Intent(getBaseContext(), MainActivity.class));
                        }
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                return builder.create();
            case GAME_REQUEST_DIALOG:
                builder.setTitle(bundle.get(General.PUZZLE_NAME).toString());
                builder.setItems(R.array.gamerequest, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int which) {
                        if (which == 0) { // = play
                            Intent intent = new Intent(getBaseContext(), MainPlayActivity.class);
                            intent.putExtras(bundle);
                            startActivity(intent);
                        } else if (which == 1) { // = forfeit
                            forfeitGameRequest(bundle.getString(General.ID));
                            startActivity(new Intent(getBaseContext(), MainActivity.class));
                        }
                    }
                });
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                return builder.create();
            case YOU_SURE_DELETE_DIALOG:
                builder.setIcon(android.R.drawable.ic_menu_delete);
                builder.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.cancel();
                    }
                });
                builder.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        deleteCreatedPuzzle(bundle.getInt(General.ID));
                        dialogInterface.dismiss();
                    }
                });
                builder.setTitle(R.string.delete_puzzle);
                builder.setMessage(R.string.delete_puzzle_message);
                return builder.create();
            case NEED_INTERNET_DIALOG:
                builder.setMessage(R.string.updates_need_internet);
                builder.setNegativeButton(R.string.to_settings, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        startActivity(new Intent(Settings.ACTION_WIRELESS_SETTINGS));
                    }
                });
                builder.setPositiveButton(R.string.use_local, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        startUpdates(false);
                    }
                });
                builder.setCancelable(false);
                return builder.create();
            case PROGRESS_DIALOG:
                ProgressDialog dialog = new ProgressDialog(MainActivity.this, ProgressDialog.STYLE_SPINNER);
                dialog.setIndeterminate(true);
                dialog.setMessage(getResources().getString(R.string.updating));
                return dialog;
            case PUBLISH_PUZZLE:
                builder.setMessage(R.string.publish_puzzle_message);
                builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        dialogInterface.dismiss();
                    }
                });
                builder.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        IntentFilter intentFilter = new IntentFilter();
                        intentFilter.addAction(UpdaterService.PUBLISH_PUZZLE_DONE);
                        registerReceiver(uploadReceiver, intentFilter);
                        receiverRegistered = true;
                        Intent intent = new Intent(getBaseContext(), UpdaterService.class);
                        intent.putExtra(General.UPDATER_ACTION, CreateActivity.PUBLISH_PUZZLE);
                        int puzzleId = bundle.getInt(General.ID);
                        intent.putExtra(General.ID, puzzleId);
                        // Never been uploaded or not
                        intent.putExtra(General.LOCAL_PUZZLE, helper.getCoAuthors(puzzleId).size() == 0);
                        ConnectivityManager conMgr = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
                        // ARE WE CONNECTED TO THE INTERNET??
                        if (!(conMgr.getActiveNetworkInfo() != null && conMgr.getActiveNetworkInfo().isAvailable()
                                && conMgr.getActiveNetworkInfo().isConnected())) {
                            showDialog(NEED_INTERNET_DIALOG);
                            dialogInterface.dismiss();
                        } else {
                            startService(intent);
                        }
                    }
                });
                return builder.create();
        }
        return null;
    }

    private void fillListViews() {
        fetchCreatedPuzzles();
        fetchGameRequests();
        fillLatestPuzzlesView();
        // Set Item click listeners
        setOnItemClickListeners();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // Handle the back button on MainActivity
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            dismissDialog(PROGRESS_DIALOG);
            return true;
        } else if (keyCode == KeyEvent.KEYCODE_HOME) {
            dismissDialog(PROGRESS_DIALOG);
            return super.onKeyDown(keyCode, event);
        } else {
            return super.onKeyDown(keyCode, event);
        }
    }

    private void forfeitGameRequest(String id) {
        ContentValues cv = helper.getGameRequest(id);
        if (cv.containsKey(General.ID)) {
            ClientResource client = new ClientResource(General.SERVER_GAME + id + "," + cv.getAsInteger(General.ID) +
                    "," + cv.getAsInteger(General.CHALLENGER_SCORE));
            client.put(new EmptyRepresentation());
        }
        helper.deleteGameRequest(id);
    }

    private void deleteLatestPuzzle(String id) {
        helper.deleteLatestPuzzle(id);
        fillLatestPuzzlesView();
    }

    private void deleteCreatedPuzzle(int id) {
        if (helper.isPublished(id) || (helper.hasCollaborators(id) && helper.isMainAuthor(id))) {
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UpdaterService.DELETE_PUZZLE_DONE);
            registerReceiver(uploadReceiver, intentFilter);
            receiverRegistered = true;

            Intent intent = new Intent(getBaseContext(), UpdaterService.class);
            intent.putExtra(General.ID, id);
            intent.putExtra(General.UPDATER_ACTION, DELETE_PUZZLE);
            startService(intent);
        } else {
            helper.deletePuzzle(id);
            fetchCreatedPuzzles();
            setOnItemClickListeners();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.main_options, menu);
        menu.findItem(R.id.help_menu_item).setIntent(new Intent(this, HelpActivity.class));
        menu.findItem(R.id.settings_menu_item).setIntent(new Intent(this, SettingsActivity.class));
        menu.findItem(R.id.friends_menu_item).setIntent(new Intent(this, FriendsActivity.class));
        menu.findItem(R.id.search_menu_item).setIntent(new Intent(this, SearchActivity.class));
        menu.findItem(R.id.update_menu_item).setIntent(new Intent(this, MainActivity.class));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        super.onOptionsItemSelected(item);
        if (item.getItemId() == R.id.update_menu_item) {
            startUpdates(false);
            Toast.makeText(getBaseContext(), R.string.need_internet, Toast.LENGTH_SHORT).show();
        } else if(item.getItemId() == R.id.search_menu_item) {
            Toast.makeText(getBaseContext(), R.string.need_internet, Toast.LENGTH_SHORT).show();
        } else {
            startActivity(item.getIntent());
        }
        return true;
    }

    @Override
    protected void onRestart() {
        setOnItemClickListeners();
        super.onRestart();
    }

    @Override
    protected void onStart() {
        main.setCurrentTabByTag(defaultPreferences.getString(General.MAIN_TAB, "createdTab"));
        super.onStart();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (receiverRegistered) {
            receiverRegistered = false;
            unregisterReceiver(uploadReceiver);
        }
    }

    private void initTabs() {
        main.setup();
        TabHost.TabSpec createdTab = main.newTabSpec("createdTab");                  // btn_star_big_off?
        createdTab.setIndicator(getResources().getString(R.string.created), getResources().getDrawable(android.R.drawable.btn_star_big_on));
        createdTab.setContent(R.id.ListView_Created);
        main.addTab(createdTab);

        TabHost.TabSpec latestTab = main.newTabSpec("latestTab");
        latestTab.setIndicator(getResources().getString(R.string.latest), getResources().getDrawable(android.R.drawable.btn_star_big_on));
        latestTab.setContent(R.id.ListView_Latest);
        main.addTab(latestTab);

        TabHost.TabSpec requestTab = main.newTabSpec("requestTab");
        requestTab.setIndicator(getResources().getString(R.string.request), getResources().getDrawable(android.R.drawable.btn_star_big_on));
        requestTab.setContent(R.id.ListView_Request);
        main.addTab(requestTab);
    }

    private void fillLatestPuzzlesView() {
        ArrayList<ContentValues> latestList = helper.getLatestPuzzles();
        adapter = new LazyAdapter(this, latestList);
        latestView.setAdapter(adapter);
    }

    private void fetchCreatedPuzzles() {
        ArrayList<ContentValues> createdList = helper.getCreatedPuzzles(settings.getString(General.XML_TAG_NICKNAME, ""));
        adapter = new LazyAdapter(this, createdList);
        createdView.setAdapter(adapter);
    }

    private void fetchGameRequests() {
        ArrayList<ContentValues> requestList = helper.getGameRequests();
        adapter = new LazyAdapter(this, requestList);
        requestsView.setAdapter(adapter);
    }

    private BroadcastReceiver uploadReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
            switch (action) {
                case FETCH_CREATED_PUZZLES:
                case FETCH_GAME_REQUESTS:
                    startUpdates(false);
                    Toast.makeText(getBaseContext(), R.string.created_puzzles_updated, Toast.LENGTH_SHORT).show();
//                    Toast.makeText(getBaseContext(), R.string.game_requests_updated, Toast.LENGTH_SHORT).show();
                    dismissDialog(PROGRESS_DIALOG);
                    break;
                case CreateActivity.PUBLISH_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.published, Toast.LENGTH_SHORT).show();
                    startUpdates(false);
                    break;
                case ResultActivity.RATE_PUZZLE:
                    Toast.makeText(getBaseContext(), R.string.puzzle_rated, Toast.LENGTH_SHORT).show();
                    startUpdates(false);
                    break;
                case DELETE_PUZZLE:
                    fillListViews();
                    break;
            }
        }
    };
}