package com.master.playit.helper;

import android.app.IntentService;
import android.content.ContentValues;
import android.content.Intent;
import android.content.SharedPreferences;
import com.master.playit.*;
import org.restlet.data.MediaType;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.representation.EmptyRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.IOException;
import java.util.*;

public class UpdaterService extends IntentService {

    private ContentHelper helper;
    private SharedPreferences settings;

    public UpdaterService() {
        super("UpdaterService");
    }

    @Override
    public void onCreate() {
        super.onCreate();
        helper = new ContentHelper(getBaseContext());
        settings = getSharedPreferences(General.GAME_PREFERENCES, MODE_PRIVATE);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        int action = intent.getIntExtra(General.UPDATER_ACTION, 0);
        int puzzleId = intent.getIntExtra(General.ID, 0);
        XmlParser parser = new XmlParser();
        ClientResource server;
        Representation response;
        String nickname;
        ArrayList<String> authors;
        ContentValues puzzle;
        ArrayList<ContentValues> questionSet;
        try {
            DomRepresentation rep = new DomRepresentation(MediaType.APPLICATION_XML);
            Document doc = rep.getDocument();
            switch (action) {
                case CreateActivity.PUBLISH_PUZZLE:
                    boolean localPuzzle = intent.getBooleanExtra(General.LOCAL_PUZZLE, false);
                    if (localPuzzle) {
                        nickname = settings.getString(General.GAME_PREFERENCES_NICKNAME, "");
                        puzzle = helper.getCreatedPuzzle(puzzleId);
                        questionSet = helper.getQuestionSet(puzzleId);
                        XmlGeneratorHelper.generatePuzzleAndQuestionSetNode(doc, puzzle, null, nickname, questionSet);

//                        Remove to enable server communication and server publish before local
                        notifyFinished(PUBLISH_PUZZLE_DONE);
                        helper.publishPuzzleLocally(puzzleId, puzzleId);

                    } else {
                        // TODO
                    }
                    // Update puzzle to server, Uncomment to enable server communication
//                    server = new ClientResource(General.SERVER_PUBLISH + puzzleId);
//                    response = server.put(rep);
//                    int id = Integer.parseInt(response.getText());
//                    if (id != 0) {
//                        notifyFinished(PUBLISH_PUZZLE_DONE);
//                        helper.publishPuzzleLocally(puzzleId, id);
//                    }
                    break;
                case CreateActivity.COLLABORATE_PUZZLE:
                    boolean newlyCreated = intent.getBooleanExtra(General.NEWLY_CREATED, false);
                    // Saving a newly created puzzle or updating an existing puzzle
                    ArrayList<String> coAuthors = helper.getCoAuthors(puzzleId);
                    nickname = settings.getString(General.GAME_PREFERENCES_NICKNAME, "");
                    puzzle = helper.getCreatedPuzzle(puzzleId);
                    if (newlyCreated) {
                        server = new ClientResource(General.SERVER_PUZZLE);
                        XmlGeneratorHelper.generatePuzzleNode(doc, puzzle, coAuthors, nickname);
                    } else {
                        server = new ClientResource(General.SERVER_PUZZLE + puzzleId);
                        questionSet = helper.getQuestionSet(puzzleId);
                        XmlGeneratorHelper.generatePuzzleAndQuestionSetNode(doc, puzzle, coAuthors, nickname, questionSet);
                    }
                    response = server.put(rep);
                    String responseText = response.getText();
                    if (newlyCreated && !responseText.equals("0")) {
                        int newId = Integer.parseInt(responseText);
                        helper.updatePuzzleId(puzzleId, newId);
                        SharedPreferences.Editor editor = settings.edit();
                        editor.putInt(General.GAME_PREFERENCES_CURRENT_PUZZLE, newId);
                        editor.commit();
                    }
                    if (!responseText.equals("0"))
                        notifyFinished(COLLABORATE_DONE);
                    else onHandleIntent(intent);
                    break;
                case QuestionSetActivity.QUESTION_SET_UPDATE:
                    doc = parser.getDocFromUrl(General.SERVER_PUZZLE + puzzleId + General.SERVER_QUESTIONSET);
                    NodeList nodes = doc.getElementsByTagName(General.QUESTIONSET);
                    ArrayList<ContentValues> list = new ArrayList<ContentValues>();
                    for (int i = 0; i < nodes.getLength(); i++) {
                        ContentValues cv = new ContentValues();
                        cv.put(General.XML_TAG_PUZZLE_ID, puzzleId);
                        cv.put(General.XML_TAG_SET_ID, parser.getValue((Element) nodes.item(i), General.XML_TAG_SET_ID));
                        cv.put(General.XML_TAG_QUESTION, parser.getValue((Element) nodes.item(i), General.XML_TAG_QUESTION));
                        cv.put(General.XML_TAG_ANSWER, parser.getValue((Element) nodes.item(i), General.XML_TAG_ANSWER));
                        cv.put(General.XML_TAG_INPUTMETHOD, parser.getValue((Element) nodes.item(i), General.XML_TAG_INPUTMETHOD));
                        list.add(cv);
                    }
                    // TODO: Have a number on each question set, id in order of their arrival
                    helper.addQuestionSet(list);
                    list = helper.getQuestionSet(puzzleId);
                    server = new ClientResource(General.SERVER_PUZZLE + puzzleId + General.SERVER_QUESTIONSET);
                    XmlGeneratorHelper.generateQuestionSetNodes(doc, puzzleId, list);
                    response = server.put(rep);

                    // TODO: Save question set
                    notifyFinished(QUESTION_SET_DONE);
                    break;
                case MainActivity.FETCH_CREATED_PUZZLES:
                    Document createdPuzzles = parser.getDocFromUrl(General.SERVER_PUZZLES +
                            settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0) + General.SERVER_EDITABLE);
                    ArrayList<ContentValues> createdList = parser.getPuzzles(createdPuzzles);
                    helper.saveCreatedPuzzles(createdList, settings.getString(General.GAME_PREFERENCES_NICKNAME, ""));
                    notifyFinished(FETCH_CREATED_DONE);
                    break;
                case MainActivity.FETCH_GAME_REQUESTS:
                    Document gameRequests = parser.getDocFromUrl(General.SERVER_GAME +
                            settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0));
                    NodeList requestNodes = gameRequests.getElementsByTagName(General.XML_TAG_PUZZLE);
                    ArrayList<ContentValues> requestList = parser.getGameRequests(requestNodes);

                    helper.saveGameRequests(requestList);
                    notifyFinished(FETCH_GAME_REQUEST_DONE);
                    break;
                case MainPlayActivity.GET_PUZZLE_TO_PLAY:
                    boolean puzzleExists = helper.puzzleExistsAndIsSaved(puzzleId);
                    if (!puzzleExists) {
                        doc = parser.getDocFromUrl(General.SERVER_PUZZLE + puzzleId);
                        Element puzzleElement = doc.getDocumentElement();
                        puzzle = parser.getPuzzle(puzzleElement);
                        authors = parser.getAuthors(puzzleElement);
                        Set authorSet = new LinkedHashSet<String>(authors);
                        authors.clear();
                        authors.addAll(authorSet);
                        questionSet = parser.getQuestionSet(puzzleElement);
                        helper.savePuzzleToPlay(puzzle, authors, questionSet, puzzleId);
                    }
                    notifyFinished(GET_PUZZLE_DONE);
                    break;
                case CreateActivity.GET_PUZZLE_TO_EDIT:
                    // When collaborating a puzzle needs to be updated anyway
                    doc = parser.getDocFromUrl(General.SERVER_PUZZLE + puzzleId + General.SERVER_EDITABLE);
                    Element puzzleElement = doc.getDocumentElement();
                    puzzle = parser.getPuzzle(puzzleElement);
                    questionSet = parser.getQuestionSet(puzzleElement);
                    helper.savePuzzleToEdit(puzzle, null, questionSet, puzzleId);
                    notifyFinished(GET_PUZZLE_EDIT_DONE);
                    break;
                case SearchActivity.GET_SEARCH_RESULTS:
                    String category = intent.getStringExtra(General.PUZZLE_CATEGORY);
                    doc = parser.getDocFromUrl(General.SERVER_PUZZLES + category);
                    ArrayList<ContentValues> puzzles = parser.getPuzzles(doc);
                    Intent newIntent = new Intent();
                    newIntent.putParcelableArrayListExtra(General.SEARCH_RESULTS, puzzles);
                    notifyFinished(GET_SEARCH_RESULTS_DONE, newIntent);
                    break;
                case ResultActivity.RATE_PUZZLE:
                    int rating = intent.getIntExtra(General.PUZZLE_RATINGGIVEN, -1);
                    server = new ClientResource(General.SERVER_PUZZLE + puzzleId + "," + rating + General.SERVER_RATING);
                    response = server.put(new EmptyRepresentation());
                    String result = response.getText();
                    if (!result.contains("-1")) {
                        notifyFinished(PUZZLE_RATED_DONE);
                        helper.saveLatestPlayedPuzzle(puzzleId, rating, result);
                    }
                    break;
                case MainActivity.DELETE_PUZZLE:
                    server = new ClientResource(General.SERVER_PUZZLE + puzzleId);
                    response = server.delete();
                    if (response.getText().toString().equals("1")) {
                        helper.deletePuzzle(puzzleId);
                        notifyFinished(DELETE_PUZZLE_DONE);
                    }
                    break;
                case FriendsActivity.FRIEND_ADDED:
                    nickname = intent.getStringExtra(General.XML_TAG_NICKNAME);

                    server = new ClientResource(General.SERVER_FRIENDS + settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0));
                    rep = new DomRepresentation(MediaType.APPLICATION_XML);
                    doc = rep.getDocument();
                    XmlGeneratorHelper.generateFriendNode(doc, nickname);
                    rep = new DomRepresentation(server.put(rep));
                    doc = rep.getDocument();

                    Element item = doc.getDocumentElement();
                    newIntent = new Intent();
                    if (item != null) {
                        String friendId = parser.getValue(item, General.XML_TAG_ID);
                        if (!friendId.equals("")) {
                            ContentValues cv = new ContentValues();
                            cv.put(General.ID, friendId);
                            cv.put(General.XML_TAG_NICKNAME, parser.getValue(item, General.XML_TAG_NICKNAME));
                            helper.addFriend(cv);
                        }
                    } else
                        intent.putExtra(General.RESPONSE, rep.getText().toString());
                    notifyFinished(FRIEND_ADDED_DONE, newIntent);
                    break;
                case ScoreActivity.FETCH_SCORES:
                    doc = parser.getDocFromUrl(General.SERVER_SCORES + puzzleId);
                    ArrayList<ContentValues> scores = parser.getScores(doc);
                    newIntent = new Intent();
                    newIntent.putParcelableArrayListExtra(General.SCORES, scores);
                    notifyFinished(FETCH_SCORES_DONE, newIntent);
                    break;
                case ResultActivity.UPLOAD_SCORE:
                    server = new ClientResource(General.SERVER_SCORES + puzzleId + "," + settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0));
                    rep = new DomRepresentation(MediaType.TEXT_XML);
                    doc = rep.getDocument();
                    int score = intent.getIntExtra(General.SCORE, 0);
                    String time = intent.getStringExtra(General.TIME);
                    if (time.length() == 4)
                        time = "0" + time;
                    XmlGeneratorHelper.generateScoreNode(doc, puzzleId, settings.getString(General.GAME_PREFERENCES_NICKNAME, ""), score, time);
                    response = server.put(rep);
                    if (!response.getText().equals("0"))
                        notifyFinished(UPLOAD_SCORES_DONE);
                    break;
                case ResultActivity.ADD_TIMES_PLAYED:
                    server = new ClientResource(General.SERVER_SCORES + puzzleId + General.SERVER_TIMES_PLAYED);
                    response = server.put(new EmptyRepresentation());
                    String text = response.getText();
                    if (text.equals("1")) {
                        helper.addTimesPlayed(puzzleId);
                        notifyFinished(ADD_TIMES_PLAYED_DONE);
                    }
                    break;
                case FriendsActivity.GET_FRIENDS:
                    int userId = settings.getInt(General.GAME_PREFERENCES_PLAYER_ID, 0);
                    server = new ClientResource(General.SERVER_FRIENDS + userId);
                    rep = new DomRepresentation(server.get());
                    doc = rep.getDocument();
                    ArrayList<ContentValues> friends = parser.getFriends(doc);
                    helper.saveFriends(friends);
                    notifyFinished(GET_FRIENDS_DONE);
                    break;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static final String PUBLISH_PUZZLE_DONE = "com.master.publish-done";
    public static final String COLLABORATE_DONE = "com.master.collaborate-done";
    public static final String QUESTION_SET_DONE = "com.master.question-set-done";
    public static final String FETCH_CREATED_DONE = "com.master.fetch-created-done";
    public static final String FETCH_GAME_REQUEST_DONE = "com.master.fetch-game-request-done";
    public static final String GET_PUZZLE_DONE = "com.master.get-puzzle-done";
    public static final String GET_PUZZLE_EDIT_DONE = "com.master.get-puzzle-edit-done";
    public static final String GET_SEARCH_RESULTS_DONE = "com.master.get-search-results-done";
    public static final String PUZZLE_RATED_DONE = "com.master.rate-puzzle";
    public static final String DELETE_PUZZLE_DONE = "com.master.delete-puzzle-done";
    public static final String FRIEND_ADDED_DONE = "com.master.friend-added-done";
    public static final String FETCH_SCORES_DONE = "com.master.fetch-scores-done";
    public static final String UPLOAD_SCORES_DONE = "com.master.upload-scores-done";
    public static final String ADD_TIMES_PLAYED_DONE = "com.master.add-times-played-done";
    public static final String GET_FRIENDS_DONE = "com.master.get-friends-done";

    private void notifyFinished(String done, Intent newIntent) {
        Intent intent = new Intent(done);
        if (done.equals(QUESTION_SET_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, QuestionSetActivity.QUESTION_SET_UPDATE);

        } else if (done.equals(COLLABORATE_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, CreateActivity.COLLABORATE_PUZZLE);

        } else if (done.equals(PUBLISH_PUZZLE_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, CreateActivity.PUBLISH_PUZZLE);

        } else if (done.equals(FETCH_CREATED_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, MainActivity.FETCH_CREATED_PUZZLES);

        } else if (done.equals(FETCH_GAME_REQUEST_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, MainActivity.FETCH_GAME_REQUESTS);

        } else if (done.equals(GET_PUZZLE_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, MainPlayActivity.GET_PUZZLE_TO_PLAY);

        } else if (done.equals(GET_PUZZLE_EDIT_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, CreateActivity.GET_PUZZLE_TO_EDIT);

        } else if (done.equals(GET_SEARCH_RESULTS_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, SearchActivity.GET_SEARCH_RESULTS);
            intent.putExtras(newIntent);

        } else if (done.equals(PUZZLE_RATED_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, ResultActivity.RATE_PUZZLE);

        } else if (done.equals(DELETE_PUZZLE_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, MainActivity.DELETE_PUZZLE);

        } else if (done.equals(FRIEND_ADDED_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, FriendsActivity.FRIEND_ADDED);
            intent.putExtras(newIntent);

        } else if (done.equals(FETCH_SCORES_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, ScoreActivity.FETCH_SCORES);
            intent.putExtras(newIntent);

        } else if (done.equals(UPLOAD_SCORES_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, ResultActivity.UPLOAD_SCORE);

        } else if (done.equals(ADD_TIMES_PLAYED_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, ResultActivity.ADD_TIMES_PLAYED);

        } else if (done.equals(GET_FRIENDS_DONE)) {
            intent.putExtra(General.UPDATER_ACTION, FriendsActivity.GET_FRIENDS);

        }
        UpdaterService.this.sendBroadcast(intent);
    }

    private void notifyFinished(String done) {
        notifyFinished(done, null);
    }
}
