package com.application;

import org.restlet.data.MediaType;
import org.restlet.ext.xml.DomRepresentation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.io.IOException;
import java.sql.*;
import java.util.List;

public class JdbcService implements Jdbc {
    private static JdbcService service = null;
    private static final String JDBC_PASSWORD = "password";
    private static final String JDBC_USER = "root";
    private static final String JDBC_URL = "jdbc:mysql://localhost/mydatabase";
    private static final String JDBC_DRIVER = "com.mysql.jdbc.Driver";

    public static JdbcService getInstance() {
        if (service == null) {
            service = new JdbcService();
        }
        return service;
    }

    private JdbcService() {
        try {
            Class.forName(JDBC_DRIVER);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    private Connection getConnection() {
        try {
            return DriverManager.getConnection(JDBC_URL, JDBC_USER, JDBC_PASSWORD);
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String savePuzzle(DomRepresentation rep, int puzzleId) {
        Connection connection = getConnection();
        try {
            String name = rep.getText("/puzzle/name");
            String category = rep.getText("/puzzle/category");
            String author = rep.getText("/puzzle/author");
            String publishDate = rep.getText("/puzzle/publishdate");
            List<Node> coauthors = rep.getNodes("/puzzle/coauthors/author");
            List<Node> questionSets = rep.getNodes("/puzzle/questionset");

            PreparedStatement prepStatement;
            if (puzzleId != 0) {
                Statement statement = connection.createStatement();
                statement.execute("DELETE FROM questionset WHERE puzzleId = " + puzzleId);
                insertQuestionSets(puzzleId, questionSets);

                statement.execute("DELETE FROM author WHERE puzzleId = " + puzzleId);
                insertCoauthors(puzzleId, coauthors);

                prepStatement = connection.prepareStatement("UPDATE puzzle SET puzzleName = ?, category = ? WHERE puzzleId = ?");
                prepStatement.setString(1, name);
                prepStatement.setString(2, category);
                prepStatement.setInt(3, puzzleId);
                prepStatement.executeUpdate();
            } else {
                if (!publishDate.equals("")) {
                    prepStatement = connection.prepareStatement("INSERT INTO puzzle (puzzleName,category,rating," +
                            "publishDate,timesPlayed) VALUES (?,?,0,?,0)",
                            Statement.RETURN_GENERATED_KEYS);
                    prepStatement.setDate(3, Date.valueOf(publishDate));
                } else {
                    prepStatement = connection.prepareStatement("INSERT INTO puzzle (puzzleName,category,rating," +
                            "publishDate,timesPlayed) VALUES (?,?,null,null,null)",
                            Statement.RETURN_GENERATED_KEYS);
                }
                // If the puzzle does not have an id the entire thing is new
                prepStatement.setString(1, name);
                prepStatement.setString(2, category);
                prepStatement.executeUpdate();
                // Getting the generated puzzleId
                ResultSet rs = prepStatement.getGeneratedKeys();
                if (rs.next()) {
                    puzzleId = rs.getInt(1);
                    if (coauthors.size() > 0) {
                        insertCoauthors(puzzleId, coauthors);
                    }
                    insertQuestionSets(puzzleId, questionSets);
                }
            }
            prepStatement = connection.prepareStatement("INSERT INTO author (userId, puzzleId, mainAuthor) " +
                    "VALUES (?,?,TRUE)");
            prepStatement.setInt(1, getUserId(author));
            prepStatement.setInt(2, puzzleId);
            prepStatement.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        try {
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return String.valueOf(puzzleId);
    }

    private void insertQuestionSets(Integer puzzleId, List<Node> questionSets) throws SQLException {
        PreparedStatement prepStatement;
        Statement statement = getConnection().createStatement();
        ResultSet rs;
        XmlParser parser = new XmlParser();
        Connection connection = getConnection();
        for (int i = 0; i < questionSets.size(); i++) {
            String setId = parser.getValue((Element) questionSets.get(i), "setId");
            int index = 1;
            if (setId.length() > 0) {
                rs = statement.executeQuery("SELECT * FROM questionset WHERE setId = " + setId);
                if (rs.next()) {
                    prepStatement = connection.prepareStatement("UPDATE questionset SET setId = ?, puzzleId = ?," +
                            "question = ?, answer = ?, inputmethod = ?");
                    prepStatement.setInt(index, Integer.parseInt(setId));
                    index++;
                } else {
                    prepStatement = connection.prepareStatement("INSERT INTO questionset (puzzleId,question,answer," +
                            "inputmethod) VALUES (?,?,?,?)", Statement.RETURN_GENERATED_KEYS);
                }
            } else {
                prepStatement = connection.prepareStatement("INSERT INTO questionset (puzzleId,question,answer," +
                        "inputmethod) VALUES (?,?,?,?)", Statement.RETURN_GENERATED_KEYS);
            }
            prepStatement.setInt(index, puzzleId);
            prepStatement.setString(index + 1, parser.getValue((Element) questionSets.get(i), "question"));
            prepStatement.setString(index + 2, parser.getValue((Element) questionSets.get(i), "answer"));
            prepStatement.setString(index + 3, parser.getValue((Element) questionSets.get(i), "inputmethod"));
            prepStatement.executeUpdate();
        }
        connection.close();
    }

    @Override
    public String saveQuestionSet(DomRepresentation toSave, int puzzleId) {
        String result = "0";
        List<Node> questionSets = toSave.getNodes("/questionset");
        try {
            insertQuestionSets(puzzleId, questionSets);
            result = "1";
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String addTimesPlayed(int puzzleId) {
        Integer response = 0;
        Connection connection = getConnection();
        try {
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT timesPlayed FROM puzzle WHERE puzzleId = " + puzzleId);
            if (rs.next()) {
                int timesPlayed = rs.getInt("timesPlayed") + 1;
                response = statement.executeUpdate("UPDATE puzzle SET timesPlayed = " + timesPlayed +
                        " WHERE puzzleId = " + puzzleId);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return response.toString();
    }

    private void insertCoauthors(Integer puzzleId, List<Node> coauthor) throws SQLException {
        PreparedStatement prepStatement;
        Connection connection = getConnection();
        for (int i = 0; i < coauthor.size(); i++) {
            int userId = getUserId(coauthor.get(0).getTextContent());
            prepStatement = connection.prepareStatement("INSERT INTO author (puzzleId,userId,mainAuthor) VALUES (?,?,FALSE)");
            prepStatement.setInt(1, puzzleId);
            prepStatement.setInt(2, userId);
            prepStatement.executeUpdate();
        }
        connection.close();
    }

    public Integer getUserId(String nickname) {
        Integer userId = 0;
        Connection connection = getConnection();
        try {
            PreparedStatement statement = connection.prepareStatement("SELECT id FROM users WHERE nickname = ?");
            statement.setString(1, nickname);
            ResultSet rs = statement.executeQuery();
            if (rs.next()) {
                userId = rs.getInt("id");
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return userId;
    }

    @Override
    public DomRepresentation getPuzzle(int puzzleId, boolean editable) {
        DomRepresentation result = null;
        Connection connection = getConnection();
        try {
            result = new DomRepresentation(MediaType.TEXT_XML);
            Statement statement = connection.createStatement();
            Statement statement2 = connection.createStatement();
            Statement statement3 = connection.createStatement();
            ResultSet rs;
            if (!editable) {
                rs = statement.executeQuery("SELECT * FROM puzzle WHERE puzzleId = " + puzzleId + " AND publishDate IS " +
                        "NOT NULL ORDER BY publishDate DESC");
            } else {
                rs = statement.executeQuery("SELECT * FROM puzzle WHERE publishDate IS NULL AND puzzleId = " + puzzleId);
            }
            ResultSet authors = statement2.executeQuery("SELECT userId, mainAuthor FROM author WHERE puzzleId = " + puzzleId);
            if (rs.next()) {
                Document doc = result.getDocument();
                Node puzzleElt = generatePuzzleNode(rs, doc, null, null, authors);
                ResultSet qs = statement3.executeQuery("SELECT * FROM questionset WHERE puzzleId =" + puzzleId);
                int count = 0;
                while (qs.next()) {
                    generateQuestionSetNode(qs, doc, puzzleElt);
                    count++;
                }
                if (count > 0) {
                    Node countElt = doc.createElement("numberOfQuestions");
                    countElt.setTextContent(String.valueOf(count));
                    puzzleElt.appendChild(countElt);
                }
                generateAuthorNodes(authors, doc, puzzleElt);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public DomRepresentation getPuzzles(String query, boolean editable) {
        DomRepresentation result = null;
        Connection connection = getConnection();
        try {
            ResultSet rs;
            PreparedStatement prepStatement;
            Statement statement;
            if (!editable) {
                // Return a list of puzzle names, categories, ratings and ids, only those published
                if (query != null) {
                    prepStatement = connection.prepareStatement("SELECT * FROM puzzle WHERE category = ? " +
                            "AND publishDate IS NOT NULL ORDER BY publishDate DESC LIMIT 0, 10");
                    prepStatement.setString(1, query);
                    rs = prepStatement.executeQuery();
                } else {
                    statement = connection.createStatement();
                    rs = statement.executeQuery("SELECT * FROM puzzle HAVING publishDate ORDER BY publishDate DESC LIMIT 0, 10");
                }
            } else {
                // Return a list of puzzle names, categories, ratings and ids, only those with query as it's author
                prepStatement = connection.prepareStatement("SELECT * FROM puzzle INNER JOIN author " +
                        "USING(puzzleId) WHERE userId = ?");
                prepStatement.setInt(1, Integer.parseInt(query));
                // TODO: Fetch all those published within the last 7 days to edit
                rs = prepStatement.executeQuery();
            }
            result = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = result.getDocument();
            Node puzzlesElt = doc.createElement("puzzles");
            doc.appendChild(puzzlesElt);
            while (rs.next()) {
                // Generates a puzzle node for each puzzle, max 10
                statement = connection.createStatement();
                Statement statement2 = connection.createStatement();
                int puzzleId = rs.getInt("puzzleId");
                ResultSet count = statement.executeQuery("SELECT COUNT(*) FROM questionset WHERE puzzleId = " + puzzleId);
                ResultSet authors = statement2.executeQuery("SELECT * FROM author WHERE puzzleId = " + puzzleId);
                count.next();
                generatePuzzleNode(rs, doc, puzzlesElt, count.getInt(1), authors);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String saveUser(DomRepresentation user) {
        // Save or create and return the generated or existing user id
        int userId = Integer.parseInt(user.getText("/user/id"));
        String nickname = user.getText("/user/nickname");
        Connection connection = getConnection();
        try {
            PreparedStatement prepStatement;
            ResultSet rs;
            if (userId > 0) {
                return String.valueOf(userId);
            }
            prepStatement = connection.prepareStatement("SELECT id FROM users WHERE nickname = ?");
            prepStatement.setString(1, nickname);
            rs = prepStatement.executeQuery();
            if (rs.next()) {
                return String.valueOf(rs.getInt("id"));
            }
            prepStatement = connection.prepareStatement("INSERT INTO users (nickname) VALUES(?)",
                    Statement.RETURN_GENERATED_KEYS);
            prepStatement.setString(1, nickname);
            prepStatement.setString(1, nickname);
            prepStatement.executeUpdate();
            rs = prepStatement.getGeneratedKeys();
            if (rs.next()) {
                userId = rs.getInt(1);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return String.valueOf(userId);
    }

    @Override
    public DomRepresentation getFriends(int userId) {
        DomRepresentation result = null;
        try {
            result = new DomRepresentation(MediaType.TEXT_XML);
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT * FROM friends WHERE userId = " + userId);
            Document doc = result.getDocument();
            Node friendsElt = doc.createElement("friends");
            doc.appendChild(friendsElt);
            while (rs.next()) {
                generateFriendsNode(rs, doc, friendsElt);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    private String getUserName(Integer userId) {
        String nickname = null;
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT nickname FROM users WHERE id = " + userId);
            rs.next();
            nickname = rs.getString("nickname");
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return nickname;
    }

    @Override
    public DomRepresentation addFriend(int userId, DomRepresentation friend) {
        String friendNickname = friend.getText("/friend/nickname");
        Integer friendId = getUserId(friendNickname);
        DomRepresentation response = null;
        try {
            if (friendId == 0) {
                if (getUserId(friendNickname) == 0) {
                    response = new DomRepresentation(MediaType.TEXT_XML);
                    Document doc = response.getDocument();
                    Node responseElt = doc.createElement("response");
                    responseElt.setTextContent("User does not exist");
                    doc.appendChild(responseElt);
                    return response;
                } else {
                    friendId = getUserId(friendNickname);
                }
            }
            Connection connection = getConnection();
            PreparedStatement statement = connection.prepareStatement("INSERT INTO friends (userId, friendId) VALUES(?,?)");
            statement.setInt(1, userId);
            statement.setInt(2, friendId);
            statement.executeUpdate();
            response = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = response.getDocument();
            ResultSet rs = statement.executeQuery("SELECT * FROM users WHERE id = " + friendId);
            if (rs.next()) {
                generateFriendNode(rs, doc);
            }
            connection.close();
            return response;
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            response = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = response.getDocument();
            Node responseElt = doc.createElement("response");
            doc.setTextContent("Friend not saved, please try again");
            doc.appendChild(responseElt);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return response;
    }

    @Override
    public String deleteFriend(int userId, DomRepresentation friend) {
        String result = "0";
        try {
            Connection connection = getConnection();
            PreparedStatement statement = connection.prepareStatement("DELETE FROM friends WHERE userId = ? AND friendId = ?");
            statement.setInt(1, userId);
            statement.setInt(2, Integer.parseInt(friend.getText("/friend/id")));
            result = String.valueOf(statement.executeUpdate());
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String updateHighscore(int puzzleId, int userId, DomRepresentation score) {
        String scoreValue = score.getText("/score/value");
        String totalTime = score.getText("/score/totalTime");
        String result = "0";
        try {
            PreparedStatement statement;
            Connection connection = getConnection();
            statement = connection.prepareStatement("SELECT score FROM highscore WHERE userId = " + userId +
                    " AND puzzleId = " + puzzleId);
            ResultSet rs = statement.executeQuery();
            if (rs.next()) {
                if (rs.getInt(1) < Integer.parseInt(scoreValue)) {
                    statement = connection.prepareStatement("UPDATE highscore SET score = ?, totalTime = ? " +
                            "WHERE userId = ? AND puzzleId = ?");
                    statement.setInt(1, Integer.parseInt(scoreValue));
                    statement.setTime(2, Time.valueOf("00:" + totalTime));
                    statement.setInt(3, userId);
                    statement.setInt(4, puzzleId);
                    result = String.valueOf(statement.executeUpdate());
                }
                result = "1";
            } else {
                statement = connection.prepareStatement("INSERT INTO highscore (score, totalTime, " +
                        "userId, puzzleId) VALUES (?,?,?,?)");
                statement.setInt(1, Integer.parseInt(scoreValue));
                statement.setTime(2, Time.valueOf("00:" + totalTime));          // hh:mm:ss
                statement.setInt(3, userId);
                statement.setInt(4, puzzleId);
                result = String.valueOf(statement.executeUpdate());
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        /*
        try {
            PreparedStatement statement = getConnection().prepareStatement("DELETE FROM highscore WHERE puzzleId NOT IN " +
                    "(SELECT * FROM highscore WHERE puzzleId = ? ORDER BY score DESC LIMIT 0, 5)");
            statement.setInt(1, puzzleId);
            statement.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
        }
         */
        return result;
    }

    @Override
    public DomRepresentation getHighscore(int puzzleId) {
        DomRepresentation result = null;
        try {
            Connection connection = getConnection();
            PreparedStatement statement = connection.prepareStatement("SELECT * FROM highscore WHERE puzzleId = ? " +
                    "ORDER BY score DESC");
            statement.setInt(1, puzzleId);
            ResultSet rs = statement.executeQuery();
            result = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = result.getDocument();
            Node scoresElt = doc.createElement("scores");
            doc.appendChild(scoresElt);
            while (rs.next()) {
                generateScoreNode(rs, doc, scoresElt);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String deleteUsers() {
        // Delete all users, for testing purposes
        String result = "0";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            statement.executeUpdate("DELETE FROM author");
            statement.executeUpdate("DELETE FROM friends");
            result = String.valueOf(statement.executeUpdate("DELETE FROM users"));
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public DomRepresentation getGameRequests(int userId, boolean challenger) {
        DomRepresentation result = null;
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs;
            if (challenger) {
                rs = statement.executeQuery("SELECT * FROM gamerequest WHERE challengerId = " + userId);
            } else {
                rs = statement.executeQuery("SELECT * FROM gamerequest WHERE receiverId = " + userId);
            }
            result = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = result.getDocument();
            Node requestsElt = doc.createElement("gameRequests");
            doc.appendChild(requestsElt);
            while (rs.next()) {
                generateRequestNode(rs, doc, requestsElt);
            }
            connection.close();
            return result;
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String addGameRequest(DomRepresentation request, boolean random) {
        int puzzleId = Integer.parseInt(request.getText("/gameRequest/puzzleId"));
        String type = request.getText("/gameRequest/type");
        int challengerId = Integer.parseInt(request.getText("/gameRequest/challengerId"));
        int challengeScore = Integer.parseInt(request.getText("/gameRequest/challengeScore"));
        int receiverId;
        String result = "0";
        try {
            Connection connection = getConnection();
            if (random) {
                Statement statement = connection.createStatement();
                ResultSet rs = statement.executeQuery("SELECT id FROM users WHERE id <> " + challengerId + " ORDER BY " +
                        "RAND() LIMIT 0,1");
                rs.next();
                receiverId = rs.getInt("id");
            } else {
                receiverId = Integer.parseInt(request.getText("/gameRequest/receiverId"));
            }
            PreparedStatement statement = connection.prepareStatement("INSERT INTO gamerequest (puzzleId, receiverId, " +
                    "challengerId, requestDate, type, challengeScore, winner, receiverScore) VALUES (?,?,?,?,?,?,null,null)", Statement.RETURN_GENERATED_KEYS);
            statement.setInt(1, puzzleId);
            statement.setInt(2, receiverId);
            statement.setInt(3, challengerId);
            statement.setDate(4, new Date(new java.util.Date().getTime()));
            statement.setString(5, type);
            statement.setInt(6, challengeScore);
            statement.executeUpdate();
            ResultSet rs = statement.getGeneratedKeys();
            if (rs.next()) {
                result = String.valueOf(rs.getInt(1));
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String deleteGameRequest(int requestId) {
        // Is deleted when the challenger has seen the result of tha challenge
        String result = "0";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            result = String.valueOf(statement.executeUpdate("DELETE FROM gamerequest WHERE id = " + requestId));
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String updateGameRequest(int requestId, int winner, int score) {
        // Update with winner and score when the challenger has played the puzzle
        String result = "0";
        Connection connection = getConnection();
        try {
            PreparedStatement statement = connection.prepareStatement("UPDATE gamerequest SET winner = ?, receiverScore = ? WHERE id = ?");
            statement.setInt(1, winner);
            statement.setInt(2, score);
            statement.setInt(3, requestId);
            result = String.valueOf(statement.executeUpdate());
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public DomRepresentation getQuestionSet(int puzzleId) {
        DomRepresentation response = null;
        try {
            response = new DomRepresentation(MediaType.TEXT_XML);
            Document doc = response.getDocument();
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT * FROM questionset WHERE puzzleId = " + puzzleId);
            while (rs.next()) {
                generateQuestionSetNode(rs, doc, null);
            }
            connection.close();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return response;
    }

    @Override
    public String deleteAllPuzzles() {
        String result = "0";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            statement.executeUpdate("DELETE FROM author");
            statement.executeUpdate("DELETE FROM questionset");
            statement.executeUpdate("DELETE FROM highscore");
            statement.executeUpdate("DELETE FROM gamerequest");
            result = String.valueOf(statement.executeUpdate("DELETE FROM puzzle"));
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String deleteUser(int userId) {
        // Delete user with id userId
        String result = "0";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            result = String.valueOf(statement.executeUpdate("DELETE FROM users WHERE id = " + userId));
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public DomRepresentation getUser(int id) {
        // Return all user fields
        DomRepresentation result = null;
        try {
            result = new DomRepresentation(MediaType.TEXT_XML);
            result.setIndenting(true);
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT * FROM users WHERE id = " + id);
            if (rs.next()) {
                Document doc = result.getDocument();
                generateUserNode(rs, doc);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String deletePuzzle(int puzzleId) {
        // Delete the puzzle corresponding to the puzzleId and all references to it, questionSets, author, coauthors
        String result = "0";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            statement.executeUpdate("DELETE FROM author WHERE puzzleId = " + puzzleId);
            statement.executeUpdate("DELETE FROM questionset WHERE puzzleId = " + puzzleId);
            statement.executeUpdate("DELETE FROM highscore WHERE puzzleId = " + puzzleId);

            result = String.valueOf(statement.executeUpdate("DELETE FROM puzzle WHERE puzzleId = " + puzzleId));
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String publishPuzzle(DomRepresentation puzzle, int puzzleId) {
        // Set publishDate to today
        String result = "0";
        try {
            PreparedStatement prepStatement;
            Connection connection = getConnection();
            if (puzzle.getText("puzzle/name").length() > 0) {
                Statement statement = connection.createStatement();
                ResultSet rs = statement.executeQuery("SELECT puzzleId FROM puzzle WHERE puzzleId = " + puzzleId);
                if (!rs.next()) {
                    puzzleId = Integer.parseInt(savePuzzle(puzzle, 0));
                }
            }
            // The puzzle has not been provided, assume that is exists
            prepStatement = connection.prepareStatement("UPDATE puzzle SET publishDate = ?, " +
                    "rating = 0, timesplayed = 0 WHERE puzzleId = ?");
            prepStatement.setDate(1, new Date(new java.util.Date().getTime()));
            prepStatement.setInt(2, puzzleId);
            if (prepStatement.executeUpdate() == 1) {
                result = String.valueOf(puzzleId);
            }
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public String ratePuzzle(int puzzleId, int rating) {
        String result = "-1";
        String played = "-1";
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery("SELECT timesPlayed, rating FROM puzzle WHERE puzzleId = " + puzzleId);
            int newRating = 0;
            int timesPlayed = 0;
            if (rs.next()) {
                timesPlayed = rs.getInt("timesPlayed");
                int oldRating = rs.getInt("rating");
                if (oldRating == 0)
                    newRating = rating;
                else
                    newRating = (timesPlayed * oldRating + rating) / (timesPlayed + 1);
            }
            result = String.valueOf(newRating);
            PreparedStatement prepStatement = connection.prepareStatement("UPDATE puzzle SET rating = ? " +
                    "WHERE puzzleId = ?");
            prepStatement.setInt(1, newRating);
            prepStatement.setInt(2, puzzleId);
            prepStatement.executeUpdate();
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return result + "," + played;
    }

    private Node generatePuzzleNode(ResultSet puzzle, Document doc, Node puzzlesElt, Integer count, ResultSet authors) throws SQLException {
        Integer puzzleId = puzzle.getInt("puzzleId");
        String name = puzzle.getString("puzzleName");
        String category = puzzle.getString("category");
        Integer rating = puzzle.getInt("rating");
        Integer timesPlayed = puzzle.getInt("timesPlayed");
        Date publishDate = puzzle.getDate("publishDate");

        Node puzzleElt = doc.createElement("puzzle");
        if (puzzlesElt == null) {
            doc.appendChild(puzzleElt);
        } else {
            puzzlesElt.appendChild(puzzleElt);
        }

        Node idElt = doc.createElement("id");
        idElt.setTextContent(puzzleId.toString());
        puzzleElt.appendChild(idElt);

        Node subjectElt = doc.createElement("name");
        subjectElt.setTextContent(name);
        puzzleElt.appendChild(subjectElt);

        Node contentElt = doc.createElement("category");
        contentElt.setTextContent(category);
        puzzleElt.appendChild(contentElt);

        Node ratingElt = doc.createElement("rating");
        ratingElt.setTextContent(rating.toString());
        puzzleElt.appendChild(ratingElt);

        Node dateElt = doc.createElement("publishdate");
        if (publishDate == null) {
            dateElt.setTextContent("");
        } else {
            dateElt.setTextContent(publishDate.toString());
        }
        puzzleElt.appendChild(dateElt);

        Node timesPlayedElt = doc.createElement("timesplayed");
        timesPlayedElt.setTextContent(String.valueOf(timesPlayed));
        puzzleElt.appendChild(timesPlayedElt);

        if (count != null) {
            Node countElt = doc.createElement("numberOfQuestions");
            countElt.setTextContent(String.valueOf(count));
            puzzleElt.appendChild(countElt);
        }

        Node coAuthorsElt = doc.createElement("coAuthors");
        puzzleElt.appendChild(coAuthorsElt);

        while (authors.next()) {
            String nickname = getUserName(authors.getInt("userId"));
            boolean mainAuthor = authors.getBoolean("mainAuthor");
            if (mainAuthor) {
                Node authorELt = doc.createElement("author");
                authorELt.setTextContent(nickname);
                puzzleElt.appendChild(authorELt);
            } else {
                Node coAuthorELt = doc.createElement("coAuthor");
                coAuthorELt.setTextContent(nickname);
                coAuthorsElt.appendChild(coAuthorELt);
            }
        }
        return puzzleElt;
    }

    private void generateQuestionSetNode(ResultSet qs, Document doc, Node puzzleElt) throws SQLException {
        Integer puzzleId = qs.getInt("puzzleId");
        Integer setId = qs.getInt("setId");
        String answer = qs.getString("answer");
        String question = qs.getString("question");
        String inputMethod = qs.getString("inputmethod");

        Node questionSetElt = doc.createElement("questionset");
        if (puzzleElt == null) {
            doc.appendChild(questionSetElt);
        } else {
            puzzleElt.appendChild(questionSetElt);
        }

        Node idElt = doc.createElement("puzzleId");
        idElt.setTextContent(puzzleId.toString());
        questionSetElt.appendChild(idElt);

        idElt = doc.createElement("setId");
        idElt.setTextContent(setId.toString());
        questionSetElt.appendChild(idElt);

        Node answerElt = doc.createElement("answer");
        answerElt.setTextContent(answer);
        questionSetElt.appendChild(answerElt);

        Node questionElt = doc.createElement("question");
        questionElt.setTextContent(question);
        questionSetElt.appendChild(questionElt);

        Node inputMethodElt = doc.createElement("inputmethod");
        inputMethodElt.setTextContent(inputMethod);
        questionSetElt.appendChild(inputMethodElt);
    }

    private void generateFriendsNode(ResultSet rs, Document doc, Node friendsElt) throws SQLException {
        Integer friendId = rs.getInt("friendId");
        String friendName = getUserName(friendId);

        Node friendElt = doc.createElement("friend");
        if (friendsElt == null) {
            doc.appendChild(friendElt);
        } else {
            friendsElt.appendChild(friendElt);
        }

        Node idElt = doc.createElement("id");
        idElt.setTextContent(String.valueOf(friendId));
        friendElt.appendChild(idElt);

        Node nameElt = doc.createElement("nickname");
        nameElt.setTextContent(String.valueOf(friendName));
        friendElt.appendChild(nameElt);

    }

    private void generateFriendNode(ResultSet rs, Document doc) throws SQLException {
        Integer friendId = rs.getInt("id");
        String friendName = rs.getString("nickname");

        Node friendElt = doc.createElement("friend");
        doc.appendChild(friendElt);

        Node idElt = doc.createElement("id");
        idElt.setTextContent(String.valueOf(friendId));
        friendElt.appendChild(idElt);

        Node nameElt = doc.createElement("nickname");
        nameElt.setTextContent(String.valueOf(friendName));
        friendElt.appendChild(nameElt);
    }

    private void generateScoreNode(ResultSet rs, Document doc, Node scoresElt) throws SQLException {
        String nickname = getUserName(rs.getInt("userId"));
        Integer score = rs.getInt("score");
        Integer puzzle = rs.getInt("puzzleId");
        String time = rs.getString("totalTime");

        Node scoreElt = doc.createElement("score");
        scoresElt.appendChild(scoreElt);

        Node idElt = doc.createElement("puzzleId");
        idElt.setTextContent(String.valueOf(puzzle));

        Node nameElt = doc.createElement("nickname");
        nameElt.setTextContent(String.valueOf(nickname));
        scoreElt.appendChild(nameElt);

        Node valueElt = doc.createElement("value");
        valueElt.setTextContent(String.valueOf(score));
        scoreElt.appendChild(valueElt);

        Node timeELt = doc.createElement("totalTime");
        timeELt.setTextContent(time);
        scoreElt.appendChild(timeELt);
    }

    private void generateRequestNode(ResultSet rs, Document doc, Node requestsElt) throws SQLException {
        Integer id = rs.getInt("id");
        Integer challengerId = rs.getInt("challengerId");
        Integer receiverId = rs.getInt("receiverId");
        Integer puzzleId = rs.getInt("puzzleId");
        Integer challengeScore = rs.getInt("challengeScore");
        Integer receiverScore = rs.getInt("receiverScore");
        Integer winnerId = rs.getInt("winner");
        Date requestDate = rs.getDate("requestDate");
        String type = rs.getString("type");

        Node requestElt = doc.createElement("gameRequest");
        requestsElt.appendChild(requestElt);

        Node requestIdElt = doc.createElement("requestId");
        requestIdElt.setTextContent(String.valueOf(id));
        requestElt.appendChild(requestIdElt);

        Node challengerElt = doc.createElement("challengerId");
        challengerElt.setTextContent(getUserName(challengerId));
        requestElt.appendChild(challengerElt);

        Node receiverElt = doc.createElement("receiverId");
        receiverElt.setTextContent(getUserName(receiverId));
        requestElt.appendChild(receiverElt);

        Node puzzleElt = doc.createElement("puzzleId");
        puzzleElt.setTextContent(String.valueOf(puzzleId));
        requestElt.appendChild(puzzleElt);

        Node puzzleNameElt = doc.createElement("name");
        puzzleNameElt.setTextContent(getPuzzleName(puzzleId));
        requestElt.appendChild(puzzleNameElt);

        Node puzzleCategoryElt = doc.createElement("category");
        puzzleCategoryElt.setTextContent(getPuzzleCategory(puzzleId));
        requestElt.appendChild(puzzleCategoryElt);

        Node dateElt = doc.createElement("requestDate");
        dateElt.setTextContent(requestDate.toString());
        requestElt.appendChild(dateElt);

        Node typeElt = doc.createElement("type");
        typeElt.setTextContent(type);
        requestElt.appendChild(typeElt);

        Node challengeScoreElt = doc.createElement("challengeScore");
        challengeScoreElt.setTextContent(String.valueOf(challengeScore));
        requestElt.appendChild(challengeScoreElt);

        Node receiverScoreElt = doc.createElement("receiverScore");
        receiverScoreElt.setTextContent(String.valueOf(receiverScore));
        requestElt.appendChild(receiverScoreElt);

        Node winnerElt = doc.createElement("winner");
        if (winnerId == 0) {
            winnerElt.setTextContent("");
        } else {
            winnerElt.setTextContent(getUserName(winnerId));
        }
        requestElt.appendChild(winnerElt);
    }

    private void generateAuthorNodes(ResultSet authors, Document doc, Node puzzleElt) throws SQLException {
        Statement statement = getConnection().createStatement();
        Node coAuthorsElt = doc.createElement("coauthors");
        puzzleElt.appendChild(coAuthorsElt);

        while (authors.next()) {
            int userId = authors.getInt("userId");
            ResultSet nicknames = statement.executeQuery("SELECT nickname FROM users WHERE id = " + userId);
            nicknames.next();
            String nickname = nicknames.getString("nickname");
            if (authors.getBoolean("mainAuthor") == true) {
                Node authorElt = doc.createElement("author");
                authorElt.setTextContent(nickname);
                puzzleElt.appendChild(authorElt);
            } else {
                Node coAuthorElt = doc.createElement("coauthor");
                coAuthorElt.setTextContent(nickname);
                coAuthorsElt.appendChild(coAuthorElt);
            }
        }
    }

    private void generateUserNode(ResultSet rs, Document doc) throws SQLException {
        Integer userId = rs.getInt("id");
        String nickname = rs.getString("nickname");

        Node userElt = doc.createElement("user");
        doc.appendChild(userElt);

        Node idElt = doc.createElement("id");
        idElt.setTextContent(userId.toString());
        userElt.appendChild(idElt);

        Node nicknameElt = doc.createElement("nickname");
        nicknameElt.setTextContent(nickname);
        userElt.appendChild(nicknameElt);
    }

    private String getPuzzleCategory(Integer puzzleId) {
        DomRepresentation puzzle = getPuzzle(puzzleId, false);
        return puzzle.getText("/puzzle/category");
    }

    private String getPuzzleName(Integer puzzleId) {
        DomRepresentation puzzle = getPuzzle(puzzleId, false);
        return puzzle.getText("/puzzle/name");
    }
}
